DROP TABLE IF EXISTS run_tile;
CREATE TABLE `run_tile` (
  `id_run_tile` BIGINT(20) UNSIGNED ZEROFILL NOT NULL AUTO_INCREMENT,
  `id_run` BIGINT(20) UNSIGNED NOT NULL,
  `tile` TINYINT(3) UNSIGNED NOT NULL,
  `position` TINYINT UNSIGNED NOT NULL,
  PRIMARY KEY (`id_run_tile`)
) ENGINE = InnoDB
CHARACTER SET utf8
COMMENT = 'table linking a tile to a run and lane position';

DROP TABLE IF EXISTS signal_mean;
CREATE TABLE `signal_mean` (
  `id_signal_mean` BIGINT(20) UNSIGNED AUTO_INCREMENT,
  `id_run` BIGINT(20) UNSIGNED NOT NULL,
  `position` TINYINT(3) UNSIGNED NOT NULL,
  `cycle` TINYINT UNSIGNED NOT NULL,
  `all_a` FLOAT(10,2) UNSIGNED NOT NULL DEFAULT 0,
  `all_c` FLOAT(10,2) UNSIGNED NOT NULL DEFAULT 0,
  `all_g` FLOAT(10,2) UNSIGNED NOT NULL DEFAULT 0,
  `all_t` FLOAT(10,2) UNSIGNED NOT NULL DEFAULT 0,
  `call_a` FLOAT(10,2) UNSIGNED NOT NULL DEFAULT 0,
  `call_c` FLOAT(10,2) UNSIGNED NOT NULL DEFAULT 0,
  `call_g` FLOAT(10,2) UNSIGNED NOT NULL DEFAULT 0,
  `call_t` FLOAT(10,2) UNSIGNED NOT NULL DEFAULT 0,
  `base_a` FLOAT(10,2) UNSIGNED NOT NULL DEFAULT 0,
  `base_c` FLOAT(10,2) UNSIGNED NOT NULL DEFAULT 0,
  `base_g` FLOAT(10,2) UNSIGNED NOT NULL DEFAULT 0,
  `base_t` FLOAT(10,2) UNSIGNED NOT NULL DEFAULT 0,
  PRIMARY KEY (`id_signal_mean`)
)
ENGINE = InnoDB
CHARACTER SET utf8;

DROP TABLE IF EXISTS tile_all;
CREATE TABLE `tile_all` (
  `id_tile_all` BIGINT(20) UNSIGNED AUTO_INCREMENT,
  `id_run_tile` BIGINT(20) UNSIGNED NOT NULL,
  `cycle` TINYINT(3) UNSIGNED NOT NULL DEFAULT 0,
  `all_a` FLOAT(10,2) UNSIGNED NOT NULL DEFAULT 0,
  `all_c` FLOAT(10,2) UNSIGNED NOT NULL DEFAULT 0,
  `all_g` FLOAT(10,2) UNSIGNED NOT NULL DEFAULT 0,
  `all_t` FLOAT(10,2) UNSIGNED NOT NULL DEFAULT 0,
  `call_a` FLOAT(10,2) UNSIGNED NOT NULL DEFAULT 0,
  `call_c` FLOAT(10,2) UNSIGNED NOT NULL DEFAULT 0,
  `call_g` FLOAT(10,2) UNSIGNED NOT NULL DEFAULT 0,
  `call_t` FLOAT(10,2) UNSIGNED NOT NULL DEFAULT 0,
  `num_a` SMALLINT UNSIGNED NOT NULL DEFAULT 0,
  `num_c` SMALLINT UNSIGNED NOT NULL DEFAULT 0,
  `num_g` SMALLINT UNSIGNED NOT NULL DEFAULT 0,
  `num_t` SMALLINT UNSIGNED NOT NULL DEFAULT 0,
  `num_x` SMALLINT UNSIGNED NOT NULL DEFAULT 0,
  PRIMARY KEY (`id_tile_all`),
  CONSTRAINT `id_run_tile` FOREIGN KEY `id_run_tile` (`id_run_tile`)
    REFERENCES `run_tile` (`id_run_tile`)
)
ENGINE = InnoDB
CHARACTER SET utf8;

DROP TABLE IF EXISTS tile_score;
CREATE TABLE `tile_score` (
  `id_tile_score` BIGINT(20) UNSIGNED NOT NULL DEFAULT NULL AUTO_INCREMENT,
  `id_run_tile` BIGINT(20) UNSIGNED NOT NULL,
  `base_count` INT UNSIGNED NOT NULL DEFAULT 0,
  `error_count` INT UNSIGNED NOT NULL DEFAULT 0,
  `blank_count` INT UNSIGNED NOT NULL DEFAULT 0,
  `unique_alignments` INT UNSIGNED NOT NULL DEFAULT 0,
  `ua_total_score` INT UNSIGNED UNSIGNED NOT NULL DEFAULT 0,
  `cycles` TINYINT(3) UNSIGNED NOT NULL DEFAULT 0,
  `rescore` TINYINT(1) NOT NULL DEFAULT 0,
  `score_version` FLOAT(5,2) NOT NULL DEFAULT 0,
  `score_date_run` DATETIME NOT NULL DEFAULT 0,
  `phagealign_version` FLOAT(5,2) NOT NULL DEFAULT 0,
  `phagealign_date_run` DATETIME NOT NULL DEFAULT 0,
  `max_blanks` SMALLINT(4) UNSIGNED NOT NULL DEFAULT 0,
  `seq_length` SMALLINT(4) UNSIGNED NOT NULL DEFAULT 0,
  `genome_file` VARCHAR(255),
  `bases_used` TEXT NOT NULL,
  `qualityfilter_version` FLOAT(5,2) DEFAULT NULL,
  `qualityfilter_date_run` DATETIME DEFAULT NULL,
  `filter_criterion` TEXT DEFAULT NULL,
  PRIMARY KEY (`id_tile_score`),
  CONSTRAINT `ts_rt` FOREIGN KEY `ts_rt` (`id_run_tile`)
    REFERENCES `run_tile` (`id_run_tile`)
)
ENGINE = InnoDB
CHARACTER SET utf8;

DROP TABLE IF EXISTS cumulative_errors_by_cycle;
CREATE TABLE `cumulative_errors_by_cycle` (
  `id_cumulative_errors_by_cycle` BIGINT(20) UNSIGNED NOT NULL DEFAULT NULL AUTO_INCREMENT,
  `id_run_tile` BIGINT(20) UNSIGNED NOT NULL,
  `cycle` TINYINT(2) UNSIGNED NOT NULL,
  `one` MEDIUMINT UNSIGNED NOT NULL DEFAULT 0,
  `two` MEDIUMINT UNSIGNED NOT NULL DEFAULT 0,
  `three` MEDIUMINT UNSIGNED NOT NULL DEFAULT 0,
  `four` MEDIUMINT UNSIGNED NOT NULL DEFAULT 0,
  `five` MEDIUMINT UNSIGNED NOT NULL DEFAULT 0,
  `rescore` TINYINT(1) NOT NULL DEFAULT 0,
  PRIMARY KEY (`id_cumulative_errors_by_cycle`),
  CONSTRAINT `tscebc_rt` FOREIGN KEY `tscebc_rt` (`id_run_tile`)
    REFERENCES `run_tile` (`id_run_tile`)
)
ENGINE = InnoDB
CHARACTER SET utf8;

DROP TABLE IF EXISTS error_rate_reference_including_blanks;
CREATE TABLE `error_rate_reference_including_blanks` (
  `id_error_rate_reference_including_blanks` BIGINT(20) UNSIGNED NOT NULL DEFAULT NULL AUTO_INCREMENT,
  `id_run_tile` BIGINT(20) UNSIGNED NOT NULL,
  `really` CHAR(1) NOT NULL,
  `read_as_blank` FLOAT(5,2) UNSIGNED NOT NULL DEFAULT 0,
  `read_as_a` FLOAT(5,2) UNSIGNED NOT NULL DEFAULT 0,
  `read_as_c` FLOAT(5,2) UNSIGNED NOT NULL DEFAULT 0,
  `read_as_g` FLOAT(5,2) UNSIGNED NOT NULL DEFAULT 0,
  `read_as_t` FLOAT(5,2) UNSIGNED NOT NULL DEFAULT 0,
  `rescore` TINYINT(1) NOT NULL DEFAULT 0,
  PRIMARY KEY (`id_error_rate_reference_including_blanks`),
  CONSTRAINT `tserrrib_rt` FOREIGN KEY `tserrrib_rt` (`id_run_tile`)
    REFERENCES `run_tile` (`id_run_tile`)
)
ENGINE = InnoDB
CHARACTER SET utf8;

DROP TABLE IF EXISTS error_rate_reference_no_blanks;
CREATE TABLE `error_rate_reference_no_blanks` (
  `id_error_rate_reference_no_blanks` BIGINT(20) UNSIGNED NOT NULL DEFAULT NULL AUTO_INCREMENT,
  `id_run_tile` BIGINT(20) UNSIGNED NOT NULL,
  `really` CHAR(1) NOT NULL,
  `read_as_a` FLOAT(5,2) UNSIGNED NOT NULL DEFAULT 0,
  `read_as_c` FLOAT(5,2) UNSIGNED NOT NULL DEFAULT 0,
  `read_as_g` FLOAT(5,2) UNSIGNED NOT NULL DEFAULT 0,
  `read_as_t` FLOAT(5,2) UNSIGNED NOT NULL DEFAULT 0,
  `rescore` TINYINT(1) NOT NULL DEFAULT 0,
  PRIMARY KEY (`id_error_rate_reference_no_blanks`),
  CONSTRAINT `tserrnb_rt` FOREIGN KEY `tserrnb_rt` (`id_run_tile`)
    REFERENCES `run_tile` (`id_run_tile`)
)
ENGINE = InnoDB
CHARACTER SET utf8;

DROP TABLE IF EXISTS error_rate_relative_reference_cycle_nucleotide;
CREATE TABLE `error_rate_relative_reference_cycle_nucleotide` (
  `id_error_rate_relative_reference_cycle_nucleotide` BIGINT(20) UNSIGNED NOT NULL DEFAULT NULL AUTO_INCREMENT,
  `id_run_tile` BIGINT(20) UNSIGNED NOT NULL,
  `cycle` TINYINT(3) UNSIGNED NOT NULL,
  `read_as` CHAR(1),
  `really_a` FLOAT(4,3) UNSIGNED ZEROFILL NOT NULL DEFAULT 0,
  `really_c` FLOAT(4,3) UNSIGNED ZEROFILL NOT NULL DEFAULT 0,
  `really_g` FLOAT(4,3) UNSIGNED ZEROFILL NOT NULL DEFAULT 0,
  `really_t` FLOAT(4,3) UNSIGNED ZEROFILL NOT NULL DEFAULT 0,
  `rescore` TINYINT(1) NOT NULL DEFAULT 0,
  PRIMARY KEY (`id_error_rate_relative_reference_cycle_nucleotide`),
  CONSTRAINT `tserrrcn_rt` FOREIGN KEY `tserrrcn_rt` (`id_run_tile`)
    REFERENCES `run_tile` (`id_run_tile`)
) ENGINE = InnoDB
CHARACTER SET utf8;

DROP TABLE IF EXISTS error_rate_relative_sequence_base;
CREATE TABLE `error_rate_relative_sequence_base` (
  `id_error_rate_relative_sequence_base` BIGINT(20) UNSIGNED NOT NULL AUTO_INCREMENT,
  `id_run_tile` BIGINT(20) UNSIGNED NOT NULL,
  `read_as` CHAR(1),
  `really_a` FLOAT(5,2) UNSIGNED NOT NULL DEFAULT 0,
  `really_c` FLOAT(5,2) UNSIGNED NOT NULL DEFAULT 0,
  `really_g` FLOAT(5,2) UNSIGNED NOT NULL DEFAULT 0,
  `really_t` FLOAT(5,2) UNSIGNED NOT NULL DEFAULT 0,
  `rescore` TINYINT(1) NOT NULL DEFAULT 0,
  PRIMARY KEY (`id_error_rate_relative_sequence_base`),
  CONSTRAINT `tserrsb_rt` FOREIGN KEY `tserrsb_rt` (`id_run_tile`)
    REFERENCES `run_tile` (`id_run_tile`)
)
ENGINE = InnoDB
CHARACTER SET utf8;

DROP TABLE IF EXISTS errors_by_cycle;
CREATE TABLE `errors_by_cycle` (
  `id_errors_by_cycle` BIGINT(20) UNSIGNED NOT NULL DEFAULT NULL AUTO_INCREMENT,
  `id_run_tile` BIGINT(20) UNSIGNED NOT NULL,
  `cycle` TINYINT(3) UNSIGNED NOT NULL DEFAULT 0,
  `error_percentage` FLOAT(5,2) UNSIGNED NOT NULL DEFAULT 0,
  `blank_percentage` FLOAT(5,2) UNSIGNED NOT NULL DEFAULT 0,
  `rescore` TINYINT(1) NOT NULL DEFAULT 0,
  PRIMARY KEY (`id_errors_by_cycle`),
  CONSTRAINT `tsebc_rt` FOREIGN KEY `tsebc_rt` (`id_run_tile`)
    REFERENCES `run_tile` (`id_run_tile`)
)
ENGINE = InnoDB
CHARACTER SET utf8;

DROP TABLE IF EXISTS errors_by_cycle_and_nucleotide;
CREATE TABLE `errors_by_cycle_and_nucleotide` (
  `id_errors_by_cycle_and_nucleotide` BIGINT(20) UNSIGNED NOT NULL DEFAULT NULL AUTO_INCREMENT,
  `id_run_tile` BIGINT(20) UNSIGNED NOT NULL,
  `cycle` TINYINT(3) UNSIGNED NOT NULL,
  `read_as` CHAR(1),
  `really_a` INT UNSIGNED NOT NULL DEFAULT 0,
  `really_c` INT UNSIGNED NOT NULL DEFAULT 0,
  `really_g` INT UNSIGNED NOT NULL DEFAULT 0,
  `really_t` INT UNSIGNED NOT NULL DEFAULT 0,
  `rescore` TINYINT(1) NOT NULL DEFAULT 0,
  PRIMARY KEY (`id_errors_by_cycle_and_nucleotide`),
  CONSTRAINT `tsebcan_rt` FOREIGN KEY `tsebcan_rt` (`id_run_tile`)
    REFERENCES `run_tile` (`id_run_tile`)
)
ENGINE = InnoDB
CHARACTER SET utf8;

DROP TABLE IF EXISTS errors_by_nucleotide;
CREATE TABLE `errors_by_nucleotide` (
  `id_errors_by_nucleotide` BIGINT(20) UNSIGNED NOT NULL DEFAULT NULL AUTO_INCREMENT,
  `id_run_tile` BIGINT(20) UNSIGNED NOT NULL,
  `read_as` CHAR(1),
  `really_a` INT UNSIGNED NOT NULL DEFAULT 0,
  `really_c` INT UNSIGNED NOT NULL DEFAULT 0,
  `really_g` INT UNSIGNED NOT NULL DEFAULT 0,
  `really_t` INT UNSIGNED NOT NULL DEFAULT 0,
  `rescore` TINYINT(1) NOT NULL DEFAULT 0,
  PRIMARY KEY (`id_errors_by_nucleotide`),
  CONSTRAINT `tseby_rt` FOREIGN KEY `tseby_rt` (`id_run_tile`)
    REFERENCES `run_tile` (`id_run_tile`)
)
ENGINE = InnoDB
CHARACTER SET utf8;

DROP TABLE IF EXISTS information_content_by_cycle;
CREATE TABLE `information_content_by_cycle` (
  `id_information_content_by_cycle` BIGINT(20) UNSIGNED NOT NULL DEFAULT NULL AUTO_INCREMENT,
  `id_run_tile` BIGINT(20) UNSIGNED NOT NULL,
  `cycle` TINYINT(3) UNSIGNED NOT NULL,
  `equiv_info` FLOAT(10,2) UNSIGNED NOT NULL DEFAULT 0,
  `align` INT UNSIGNED NOT NULL DEFAULT 0,
  `total` INT UNSIGNED NOT NULL DEFAULT 0,
  `rescore` TINYINT(1) NOT NULL DEFAULT 0,
  PRIMARY KEY (`id_information_content_by_cycle`),
  CONSTRAINT `tsicbc_rt` FOREIGN KEY `tsicbc_rt` (`id_run_tile`)
    REFERENCES `run_tile` (`id_run_tile`)
)
ENGINE = InnoDB
CHARACTER SET utf8
COMMENT = 'bases so far should be calculated';

DROP TABLE IF EXISTS log_likelihood;
CREATE TABLE `log_likelihood` (
  `id_log_likelihood` BIGINT(20) UNSIGNED NOT NULL AUTO_INCREMENT,
  `id_run_tile` BIGINT(20) UNSIGNED NOT NULL,
  `cycle` TINYINT(3) UNSIGNED NOT NULL,
  `read_as` CHAR(1),
  `really_a` SMALLINT(4) NOT NULL,
  `really_c` SMALLINT(4) NOT NULL,
  `really_g` SMALLINT(4) NOT NULL,
  `really_t` SMALLINT(4) NOT NULL,
  `rescore` TINYINT(1) NOT NULL DEFAULT 0,
  PRIMARY KEY (`id_log_likelihood`),
  CONSTRAINT `tsll_rt` FOREIGN KEY `tsll_rt` (`id_run_tile`)
    REFERENCES `run_tile` (`id_run_tile`)
)
ENGINE = InnoDB
CHARACTER SET utf8;

DROP TABLE IF EXISTS most_common_blank_pattern;
CREATE TABLE `most_common_blank_pattern` (
  `id_most_common_blank_pattern` BIGINT(20) UNSIGNED NOT NULL DEFAULT NULL AUTO_INCREMENT,
  `id_run_tile` BIGINT(20) UNSIGNED NOT NULL,
  `rank` TINYINT(2) UNSIGNED NOT NULL DEFAULT 1,
  `occurrence` MEDIUMINT UNSIGNED NOT NULL DEFAULT 1,
  `pattern` VARCHAR(255),
  `number_of_sequences` MEDIUMINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'Should include all sequence at this rank, if multiple pattern entries',
  `rescore` TINYINT(1) NOT NULL DEFAULT 0,
  PRIMARY KEY (`id_most_common_blank_pattern`),
  CONSTRAINT `tsmcbp_rt` FOREIGN KEY `tsmcbp_rt` (`id_run_tile`)
    REFERENCES `run_tile` (`id_run_tile`)
)
ENGINE = InnoDB
CHARACTER SET utf8
COMMENT = 'N=any non-blank character';

DROP TABLE IF EXISTS most_common_word;
CREATE TABLE `most_common_word` (
  `id_most_common_word` BIGINT(20) UNSIGNED NOT NULL DEFAULT NULL AUTO_INCREMENT,
  `id_run_tile` BIGINT(20) UNSIGNED NOT NULL,
  `rank` TINYINT(2) UNSIGNED NOT NULL DEFAULT 1,
  `occurrence` MEDIUMINT UNSIGNED NOT NULL DEFAULT 1,
  `word` VARCHAR(255),
  `number_of_sequences` MEDIUMINT UNSIGNED NOT NULL DEFAULT 1 COMMENT 'Should include all sequence at this rank, if multiple word entries',
  `rescore` TINYINT(1) NOT NULL DEFAULT 0,
 PRIMARY KEY (`id_most_common_word`),
  CONSTRAINT `tsmcw_rt` FOREIGN KEY `tsmcw_rt` (`id_run_tile`)
    REFERENCES `run_tile` (`id_run_tile`)
)
ENGINE = InnoDB
CHARACTER SET utf8
COMMENT = 'two or less blanks';

DROP TABLE IF EXISTS run_config;
CREATE TABLE `run_config` (
  `id_run_config` BIGINT(20) UNSIGNED NOT NULL AUTO_INCREMENT,
  `id_run` BIGINT(20) NOT NULL,
  `config_text` LONGTEXT DEFAULT NULL,
  PRIMARY KEY (`id_run_config`)
)
ENGINE = InnoDB
CHARACTER SET utf8
COMMENT = 'store of internally used additional config';

DROP TABLE IF EXISTS swift_report;
CREATE TABLE `swift_report` (
  `id_swift_report` BIGINT(20) UNSIGNED NOT NULL DEFAULT NULL AUTO_INCREMENT,
  `id_run_tile` BIGINT(20) UNSIGNED NOT NULL,
  `pf_bases` MEDIUMINT UNSIGNED NOT NULL DEFAULT 0,
  `pf_score` MEDIUMINT UNSIGNED NOT NULL DEFAULT 0,
  `pf_reads` MEDIUMINT UNSIGNED NOT NULL DEFAULT 0,
  `pf_unique` MEDIUMINT UNSIGNED NOT NULL DEFAULT 0,
  `npf_bases` MEDIUMINT UNSIGNED NOT NULL DEFAULT 0,
  `npf_score` MEDIUMINT UNSIGNED NOT NULL DEFAULT 0,
  `npf_reads` MEDIUMINT UNSIGNED NOT NULL DEFAULT 0,
  `npf_unique` MEDIUMINT UNSIGNED NOT NULL DEFAULT 0,
  PRIMARY KEY (`id_swift_report`),
  CONSTRAINT `sr_rt` FOREIGN KEY `sr_rt` (`id_run_tile`)
    REFERENCES `run_tile` (`id_run_tile`)
)
ENGINE = InnoDB
CHARACTER SET utf8;

DROP TABLE IF EXISTS swift_pf_errors_per_cycle;
CREATE TABLE `swift_pf_errors_per_cycle` (
  `id_swift_pf_errors_per_cycle` BIGINT(20) UNSIGNED NOT NULL DEFAULT NULL AUTO_INCREMENT,
  `id_run_tile` BIGINT(20) UNSIGNED NOT NULL,
  `cycle` TINYINT(3) UNSIGNED NOT NULL DEFAULT 0,
  `errors` SMALLINT(5) UNSIGNED NOT NULL DEFAULT 0,
  PRIMARY KEY (`id_swift_pf_errors_per_cycle`),
  CONSTRAINT `spepc_rt` FOREIGN KEY `spepc_rt` (`id_run_tile`)
    REFERENCES `run_tile` (`id_run_tile`)
)
ENGINE = InnoDB
CHARACTER SET utf8;

DROP TABLE IF EXISTS swift_npf_errors_per_cycle;
CREATE TABLE `swift_npf_errors_per_cycle` (
  `id_swift_npf_errors_per_cycle` BIGINT(20) UNSIGNED NOT NULL AUTO_INCREMENT,
  `id_run_tile` BIGINT(20) UNSIGNED NOT NULL,
  `cycle` TINYINT(3) UNSIGNED NOT NULL DEFAULT 0,
  `errors` SMALLINT(5) UNSIGNED NOT NULL DEFAULT 0,
  PRIMARY KEY (`id_swift_npf_errors_per_cycle`),
  CONSTRAINT `snepc_rt` FOREIGN KEY `snepc_rt` (`id_run_tile`)
    REFERENCES `run_tile` (`id_run_tile`)
)
ENGINE = InnoDB
CHARACTER SET utf8;

DROP TABLE IF EXISTS swift_all_intensities_per_cycle_per_base;
CREATE TABLE `swift_all_intensities_per_cycle_per_base` (
  `id_swift_all_intensities_per_cycle_per_base` BIGINT(20) UNSIGNED NOT NULL DEFAULT NULL AUTO_INCREMENT,
  `id_run_tile` BIGINT(20) UNSIGNED NOT NULL,
  `cycle` TINYINT(3) UNSIGNED NOT NULL DEFAULT 0,
  `a` FLOAT(7,3) UNSIGNED NOT NULL DEFAULT 0,
  `c` FLOAT(7,3) UNSIGNED NOT NULL DEFAULT 0,
  `g` FLOAT(7,3) UNSIGNED NOT NULL DEFAULT 0,
  `t` FLOAT(7,3) UNSIGNED NOT NULL DEFAULT 0,
  PRIMARY KEY (`id_swift_all_intensities_per_cycle_per_base`),
  CONSTRAINT `saipcpb_rt` FOREIGN KEY `saipcpb_rt` (`id_run_tile`)
    REFERENCES `run_tile` (`id_run_tile`)
)
ENGINE = InnoDB
CHARACTER SET utf8;

DROP TABLE IF EXISTS swift_called_intensities_per_cycle_per_base;
CREATE TABLE `swift_called_intensities_per_cycle_per_base` (
  `id_swift_called_intensities_per_cycle_per_base` BIGINT(20) UNSIGNED NOT NULL DEFAULT NULL AUTO_INCREMENT,
  `id_run_tile` BIGINT(20) UNSIGNED NOT NULL,
  `cycle` TINYINT(3) UNSIGNED NOT NULL DEFAULT 0,
  `a` FLOAT(7,3) UNSIGNED NOT NULL DEFAULT 0,
  `c` FLOAT(7,3) UNSIGNED NOT NULL DEFAULT 0,
  `g` FLOAT(7,3) UNSIGNED NOT NULL DEFAULT 0,
  `t` FLOAT(7,3) UNSIGNED NOT NULL DEFAULT 0,
  PRIMARY KEY (`id_swift_called_intensities_per_cycle_per_base`),
  CONSTRAINT `scipcpb_rt` FOREIGN KEY `scipcpb_rt` (`id_run_tile`)
    REFERENCES `run_tile` (`id_run_tile`)
)
ENGINE = InnoDB
CHARACTER SET utf8;
