#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-10-06
# Last Modified: $Date: 2008-10-07 14:51:04 +0100 (Tue, 07 Oct 2008) $
# Id:            $Id: loader.pm 3168 2008-10-07 13:51:04Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/lib/npg_qc/api/analysis/loader.pm $
#
package npg_qc::api::analysis::loader;
use strict;
use warnings;
use English qw(-no_match_vars);
use Carp;
use npg::api::run;
use srpipe::util;
use DBI; # ARGH!
use srpipe::log;
use Readonly;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3168 $ =~ /(\d+)/mx; $r; };

Readonly our $YEAR_MODIFIER  => 2000;
Readonly our $MIN_TILE_COUNT => 20;
Readonly our $RAW_CHASTITY   => -1;

sub new {
  my ($class, $ref) = @_;
  $ref ||= {};
  bless $ref, $class;
  return $ref;
}

sub util {
  my ($self, $val) = @_;
  if($val) {
    $self->{util} = $val;
  }

  $self->{util} ||= srpipe::util->new({log => $self->{log}});
  return $self->{util};
}

sub log { ## no critic
  my ($self, @args) = @_;
  $self->{log} ||= srpipe::log->new();
  return $self->{log}->out("summary=$self ", @args);
}

sub dbh {
  my ($self, $val) = @_;
  if($val) {
    $self->{dbh} = $val;
  }

  if(!$self->{dbh}) {
    carp q(Warning: Direct database connections arent a good idea from sanger-pipeline);
    $self->{dbh} = DBI->connect(
				'DBI:mysql:database=npgqcp;host=mcs3a;port=3303',
				'npgqcrw',
				'Kf8m/KY4',
				{
				 RaiseError => 1,
				 AutoCommit => 0,
				});
  }
  return $self->{dbh};
}

sub process_run {
  my ($self, $run_folder) = @_;
  my ($dtfolder) = $run_folder =~ m{([a-z\d_\-\./,]+)}mix;

  my ($y, $m, $d, $id_run) = $dtfolder =~ /(\d\d)(\d\d)(\d\d)_IL\d+_(\d+)/mx;
  if(!$id_run) {
    croak "Unable to determine run name from $dtfolder - is it a tracked run and is your path fully qualified?";
  }

  my $date = sprintf q(%4d-%02d-%02d), $y+$YEAR_MODIFIER, $m, $d;

  carp "Processing run $id_run";

#  if(!scalar @qcalresults) {
#    croak qq(Error! No files matching s_d+_qcalreport.txt inside $dtfolder!\n);
#  }

  my $dbh = $self->dbh();

  eval {
    #########
    # Scrape per-lane yield,clusters(raw),clusters(PF) and friends from Summary.htm
    #
    my $read_summary = $self->process_summary("$dtfolder/Summary.htm");
    my $summary_keys = [sort keys %{$read_summary->{read1}->{1}}];

    if(!scalar @{$summary_keys}) {
      croak q(No data processed. Skipping);
    }

    my $run = npg::api::run->new({
				  id_run => $id_run,
				 });
    my $run_pair = $run->run_pair();
    my @runs = ($run);
    if($run_pair && $run_pair->id_run()) {
      push @runs, $run_pair;
    }

    my $last_insert_ids = {};

    for my $read (qw(read1 read2)) {
      my $run = shift @runs;
      if(!$run) {
	next;
      }
      my $lane_summary = $read_summary->{$read};
      $dbh->do(q(UPDATE analysis SET iscurrent=0 WHERE id_run=?), {}, $run->id_run());
      $dbh->do(q(INSERT INTO analysis(id_run,date,folder,iscurrent) VALUES(?,?,?,?)), {}, $run->id_run(), $date, $dtfolder, 1);
      my $id_analysis = $dbh->selectall_arrayref(q(SELECT LAST_INSERT_ID()))->[0]->[0];

      for my $lane_position (keys %{$lane_summary}) {

	if($lane_summary->{$lane_position}->{tile_count} < $MIN_TILE_COUNT) {
	  carp "Not enough tiles for lane $lane_position. Not loading";
	  next;
	}

	my $q = qq(INSERT INTO analysis_lane(id_analysis,id_run,position,@{[join q(,), @{$summary_keys}]})
                   VALUES(?,?,?@{[q(,?)x(scalar @{$summary_keys})]}));

	my @values = ($id_analysis,
		      $run->id_run(), $lane_position,
		      map { $lane_summary->{$lane_position}->{$_}||0 } @{$summary_keys},);
	eval {
	  my $sth = $dbh->prepare($q);
	  $sth->execute(@values);
	} or do {
	  my $str = join "\n", map { qq($_,').(shift @values).q(') } @{$summary_keys};
	  croak qq(Error updating database for $dtfolder/Summary.htm, lane $lane_position:\n$EVAL_ERROR\n$q\n$str\n);
	};

        $last_insert_ids->{"$read$lane_position"} = $dbh->selectall_arrayref(q(SELECT LAST_INSERT_ID()))->[0]->[0];
      }
    }

    $self->process_run_qcalreports($dtfolder, $last_insert_ids);
    1;
  } or do {
    $dbh->rollback();
    croak $EVAL_ERROR;
  };
  $dbh->commit();

  return;
}

sub process_run_qcalreports {
  my ($self, $dtfolder, $last_insert_ids) = @_;
  my $dbh = $self->dbh();
  opendir my $dh, $dtfolder or (carp "Cannot open '$dtfolder'" && return);
  my @qcalresults = grep { /s_\d+_\d+_qcalreport\.txt/mx } sort readdir $dh;
  closedir $dh or croak qq(Cannot close '$dtfolder');

  #########
  # Now process per-lane s_X_qcalreport.txt files
  #
  for my $qcal (@qcalresults) {
    my ($raw, $filtered) = $self->process_qcal("$dtfolder/$qcal");

    my ($lane_position, $read)  = $qcal =~ /s_(\d+)_(\d+)_qcalreport/mx;

    my $id_analysis_lane = $last_insert_ids->{"read$read$lane_position"};

    if(!$id_analysis_lane) {
      $self->log("$lane_position for '$dtfolder/$qcal' has no id_analysis_lane! Skipping!");
      next;
    }

    my $q = q(INSERT INTO analysis_lane_qcal(id_analysis_lane,chastity,qv,cum_error,cum_bases,
                          cum_perc_error,cum_perc_total,error,bases,perc_error,
                          perc_total,exp_perc_error)
              VALUES(?,?,?,?,?,?,?,?,?,?,?,?));

    my $sth = $dbh->prepare($q);
    for my $row (@{$raw}) {
      $sth->execute($id_analysis_lane, @{$row});
    }

    for my $row (@{$filtered}) {
      $sth->execute($id_analysis_lane, @{$row});
    }
  }
  return;
}


sub process_qcal {
  my ($self, $fn) = @_;

  my ($dtfn) = $fn =~ m{([a-z\d_\-\./]+)}mix;

  local $RS = "\n\n";
  open my $fh, q(<), $dtfn or croak qq(Cannot open '$dtfn');
  my $rev      = <$fh>;
  my $tilelist = <$fh>;
  my $tiles    = <$fh>;
  my $raw      = $self->read_qcal_table($fh);
  my $filtered = $self->read_qcal_table($fh);

  close $fh or croak "Closing $dtfn: $ERRNO";

#  return ($tile_count, $raw, $filtered);
  return ($raw, $filtered);
}

sub read_qcal_table {
  my ($self, $fh) = @_;

  local $RS = "\n";
  my ($tmp);
  my $chastity = 0;

  while(my $tmp = <$fh>) {
    #########
    # read up to table header
    #
    if($tmp =~ /Results\ for\ (raw|quality\ filtered)?\ ?data/mx) {
      if($tmp =~ /raw/mx) {
	$chastity = $RAW_CHASTITY; # could be undef here but data-mining selects on NULL are inefficient later on
      }

      if($tmp =~ /CHASTITY/mx) {
	($chastity) = $tmp =~ /CHASTITY>=([\d\.]+)/mx;
      }
      last;
    }
  }

  $tmp = <$fh>; # - - - - - -
  $tmp = <$fh>; # QV      this QV or higher               |       this QV only                    |       expected
  $tmp = <$fh>; #         # error # bases % error % total |       # error # bases % error % total |       % error
  $tmp = <$fh>; # -       -       -       -       -       -       -       -       -       -       -       -

  my $table = [];
  while(my $line = <$fh>) {
    if($line eq "\n") {
      last;
    }

    chomp $line;
    my ($qval,
	$cum_error, $cum_bases, $cum_perc_error, $cum_perc_total,
	$dummy_1,
	$error_rate, $base_count, $perc_error, $perc_total,
	$dummy_2,
	$exp_error) = map { (/\d+/mx)?$_+0:$_; } split /\t/mx, $line;

    push @{$table}, [$chastity, $qval,
		     $cum_error, $cum_bases, $cum_perc_error, $cum_perc_total,
		     $error_rate, $base_count, $perc_error, $perc_total,
		     $exp_error];
  }

  return $table;
}

sub process_summary {
  my ($self, $summaryhtm) = @_;
  my $lane_data = {
		   read1 => {},
		   read2 => {},
		  };

  open my $fh, q(<), $summaryhtm or ($self->log("Error opening $summaryhtm: $ERRNO") and return);
  local $RS  = q(<table);
  my @blocks = <$fh>;
  close $fh or ($self->log("Error closing $summaryhtm: $ERRNO") and return);

  my $read_tracking = {
		      };

  for my $blk (@blocks) {
    #########
    # lane parameters
    # It's actually almost safe to select on <td>Lane</td> here as all other blocks are <td>Lane </td>
    # but let's work with something a little more specific
    #
    if($blk =~ /<td>Num\ Tiles</smx) {
      $self->process_summary_num_tiles($lane_data->{read1}, $blk);
      $self->process_summary_num_tiles($lane_data->{read2}, $blk);
      next;
    }

    #########
    # basic summary
    #
    if($blk =~ /<td>Lane\ Yield/smx) {
      $read_tracking->{lane_yield}++;
      $self->process_summary_lane_yield($lane_data->{"read$read_tracking->{lane_yield}"}, $blk);
      next;
    }

    #########
    # extended summary
    #
    if($blk =~ m{Phasing\ Info</td>}smx) {
      $read_tracking->{phasing_info}++;
      $self->process_summary_phasing_info($lane_data->{"read$read_tracking->{phasing_info}"}, $blk);
      next;
    }
  }

  return $lane_data;
}

sub process_summary_num_tiles {
  my ($self, $lane_data, $blk) = @_;

  for my $lane_blk ($blk =~ m{(<tr>\s+<td>\d+.*?</tr>)}smxg) {
    my @various = $lane_blk =~ m{<td>(.*?)</td>}smxg;
    my $lane    = shift @various;

    if($lane !~ /^[1-8]$/mx) {
      croak "Bogus lane data!\n$lane_blk";
    }
    for my $field (qw(sample_id
                      sample_target
                      sample_type
                      length
                      filter
                      tile_count
                      tiles)) {
      my $val = shift @various;

      if($field ne 'tile_count') {
	#########
	# only keep tile_count otherwise we upset summary_keys as not all of the fields
	# captured in this loop are kept in the database
	#
	next;
      }
      $lane_data->{$lane}->{$field} = $val;
      $self->log(qq(Summary processed $val tiles for lane $lane));
    }
  }
  return;
}

sub process_summary_lane_yield {
  my ($self, $lane_data, $blk) = @_;

  for my $lane_blk ($blk =~ m{(<tr>\s+<td>\d+.*?</tr>)}smxg) {
    my @various = $lane_blk =~ m{<td>(.*?)</td>}smxg;

    my $lane = shift @various;
    for my $field (qw(lane_yield
                      clusters_raw
                      clusters_pf
                      cycle1_int_pf
                      cycle20_perc_int
                      perc_clusters_pf
                      perc_align_pf
                      align_score_pf
                      error_rate_pf)) {
      my $val = shift @various;
      if($val =~ m{\+/-}mx) {
	my ($tmp, $err) = $val =~ m{([\d\.]+)\ \+/-\ ([\d\.]+)}mx;
	$lane_data->{$lane}->{$field} = $tmp;
	$lane_data->{$lane}->{"${field}_err"} = $err;

      } else {
	$lane_data->{$lane}->{$field} = $val;
      }
    }
  }
  return;
}

sub process_summary_phasing_info {
  my ($self, $lane_data, $blk) = @_;

  for my $lane_blk ($blk =~ m{(<tr>\s+<td>\d+.*?</tr>)}smxg) {
    my @various = $lane_blk =~ m{<td>(.*?)</td>}smxg;

    my $lane = shift @various;
    for my $field (qw(clusters_tilemean_raw
                      perc_phasing
                      perc_prephasing
                      perc_error_rate_raw
                      equiv_perfect_clusters_raw
                      perc_retained
                      cycle_2_4_av_int_pf
                      cycle_2_10_av_perc_loss_pf
                      cycle_10_20_av_perc_loss_pf
                      perc_align_pf2
                      perc_error_rate_pf
                      equiv_perfect_clusters_pf)) {

      my $val = shift @various;
      if($val =~ m{\+/-}mx) {
	my ($tmp, $err) = $val =~ m{([\d\.]+)\ \+/-\ ([\d\.]+)}mx;
	$lane_data->{$lane}->{$field}         = $tmp;
	$lane_data->{$lane}->{"${field}_err"} = $err;

      } else {
	$lane_data->{$lane}->{$field} = $val;
      }
    }

    delete $lane_data->{perc_align_pf2}; # this is duplicated from the simple summary which also includes error
  }
  return;
}

1;

__END__

=head1 NAME

npg_qc::api::analysis::loader

=head1 VERSION

$LastChangedRevision: 3168 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 dbh

=head2 log

=head2 new

=head2 process_qcal

=head2 process_run

=head2 process_run_qcalreports

=head2 process_summary

=head2 process_summary_lane_yield

=head2 process_summary_num_tiles

=head2 process_summary_phasing_info

=head2 read_qcal_table

=head2 util

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Carp

=item English -no_match_vars

=item npg::api::run

=item srpipe::log

=item srpipe::util

=item strict

=item warnings

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This file is part of NPG.

NPG is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your
option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see http://www.gnu.org/licenses/ .

=cut
