#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-06-11
# Last Modified: $Date: 2008-11-18 12:00:19 +0000 (Tue, 18 Nov 2008) $
# Id:            $Id: base.pm 3504 2008-11-18 12:00:19Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/lib/npg_qc/api/base.pm $
#
package npg_qc::api::base;
use base qw(Class::Accessor);
use strict;
use warnings;
use Carp;
use Class::Std;
use npg_qc::api::util;
use Lingua::EN::Inflect qw(PL);
use English qw(-no_match_vars);
use Readonly;
use JSON;

our $VERSION     = do { my ($r) = q$LastChangedRevision: 3504 $ =~ /(\d+)/mx; $r; };
our $DEBUG       = 0;
Readonly our $MAX_RETRIES => 10;
Readonly our $RETRY_DELAY => 10;

{
  ## no critic
  my %util_of     :ATTR( init_arg => 'util',     :get<util>,     :set<util> );
  my %read_dom_of :ATTR( :get<read_dom>, :set<read_dom> );
  ## use critic

  sub fields {
    return qw();
  }

  sub large_fields {
    return qw();
  }

  sub primary_key {
    my $self = shift;
    return ($self->fields())[0];
  }

  sub init {}

  sub util {
    my $self = shift;
    $self->get_util() || $self->set_util(npg_qc::api::util->new({ useragent => undef, base_uri => $self->base_uri(), parser => undef }));
    return $self->get_util();
  }

  sub base_uri {
    my ($self, $base_uri) = @_;
    if ($base_uri) {
      $self->util->base_uri($base_uri);
    }
    return $self->util->base_uri();
  }

  sub npg_uri {
    my ($self, $npg_uri) = @_;
    if ($npg_uri) {
      $self->util->npg_uri($npg_uri);
    }
    return $self->util->npg_uri();
  }

  sub new_from_xml {
    my ($self, $pkg, $xmlfrag) = @_;

    my $obj = $pkg->new({'util' => $self->util()});

    for my $f ($obj->fields()) {
      $obj->$f($xmlfrag->getAttribute($f));
    }

    return $obj;
  }

  sub new_from_json {
    my ($self, $pkg, $json) = @_;

    if (ref$json ne 'HASH') {
      $json = from_json($json);
    }

    my $obj = $pkg->new({'util' => $self->util()});

    for my $f ($obj->fields()) {
      my $method = 'set_'.$f;
      $obj->$method($json->{$f});
    }

    return $obj;
  }

  sub hasa {
    my ($class, $attr) = @_;
    no strict 'refs'; ## no critic

    if(ref $attr ne 'ARRAY') {
      $attr = [$attr];
    }

    for my $single (@{$attr}) {
      my $pkg = $single;
  #carp qq(SINGLE=  $single\n);
      if(ref $single eq 'HASH') {
        ($pkg)    = values %{$single};
        ($single) = keys %{$single};
  #carp qq(SET pkg=$pkg, single=$single\n);
      }
      my $namespace = "${class}::$single";

  #carp qq(Making $namespace for $class consuming <$single> and yielding npg::api::$pkg);

      if (defined &{$namespace}) {
        next;
      }

      *{$namespace} = sub {
        my $self = shift;
        my $el   = $self->read->getElementsByTagName($single)->[0];

        return $self->new_from_xml("npg::api::$pkg", $el);
      };
    }

    return;
  }

  sub hasmany {
    my ($class, $attr) = @_;
    no strict 'refs'; ## no critic

    if(ref $attr ne 'ARRAY') {
      $attr = [$attr];
    }

    for my $single (@{$attr}) {
      my $pkg    = $single;

      if(ref $single eq 'HASH') {
        ($pkg)    = values %{$single};
        ($single) = keys %{$single};
      }

      my $plural    = PL($single);
      my $namespace = "${class}::$plural";

      if (defined &{$namespace}) {
        next;
      }
  #carp qq(Making $namespace for $class consuming <$plural/$single> and yielding npg::api::$pkg);
      *{$namespace} = sub {
        my $self = shift;
        my $el   = $self->read->getElementsByTagName($plural)->[0];

        return [map { $self->new_from_xml("npg::api::$pkg", $_); }
  	            $el->getElementsByTagName($single)];

      };
    }

    return;
  }

  sub get {
    my ($self, $field) = @_;

    if(!exists $self->{$field}) {
      $self->read();
    }

    return $self->SUPER::get($field);
  }

  sub list {
    my ($self, $filters) = @_;
    my $util       = $self->util();
    my ($obj_type) = (ref $self) =~ /([^:]+)$/mx;
    my $obj_uri    = sprintf '%s/%s', $util->base_uri(), $obj_type;

    if($filters) {
      $obj_uri .= $filters;
    }

    $DEBUG and carp qq(Fetching $obj_uri);
    my $response = $self->retry(sub {
  				  my $response = $util->get($obj_uri);
  				  if(!$response->is_success()) {
  				    croak $response->status_line();
  				  }
  				  return $response;
  				});

    return $util->parser->parse_string($response->content());
  }

  sub read { ## no critic
    my $self = shift;

    if(!$self->get_read_dom()) {
      my $util       = $self->util();
      my ($obj_type) = (ref $self) =~ /([^:]+)$/mx;
      my $obj_pk     = $self->primary_key();
      my $obj_pk_val = $self->{$obj_pk};

      if(!defined $obj_pk_val) {
        return;
      }

      my $obj_uri  = sprintf '%s/%s/%s', $util->base_uri(), $obj_type, $obj_pk_val;
      $DEBUG and carp qq(Fetching $obj_uri);
      my $response = $self->retry(sub {
  				  my $response = $util->get($obj_uri);
  				  if(!$response->is_success()) {
  				    croak $response->status_line();
  				  }
  				  return $response;
  				});

      $self->set_read_dom($util->parser->parse_string($response->content()));
      my $root = $self->get_read_dom()->getDocumentElement();

      for my $field ($self->fields()) {
        $self->{$field} = $root->getAttribute($field);
  #      print STDERR qq(Discovered $field == $self->{$field} in DOM\n);
      }
    }

    return $self->get_read_dom();
  }

  sub create {
    my $self       = shift;
    my $util       = $self->util();
    my ($obj_type) = (ref $self) =~ /([^:]+)$/mx;
    my $obj_pk     = $self->primary_key();
    my $obj_pk_val = $self->$obj_pk();

    if($obj_pk_val) {
      croak qq(Cannot create a $obj_type with an existing $obj_pk ($obj_pk_val));
    }

    my $obj_uri = sprintf '%s/%s/', $util->base_uri(), $obj_type;
    $DEBUG and carp qq(Posting $obj_uri);

    my $large_fields = { map { $_ => undef } $self->large_fields() };
    my @small_fields = grep { !exists $large_fields->{$_} } $self->fields();
    my $payload      = ['Content_Type' => 'form-data',
  		      'Content'      => [
  					 'pipeline' => 1,
  					 (map { $_ => $self->{$_} || q() } @small_fields),
  					 (map { $_ => [
  						       undef,
  						       $_,
  						       'Content_Type'   => 'binary/octet-stream',
  						       'Content_Length' => length ($self->{$_}||q()),
  						       'Content'        => $self->{$_}||q(),
  						      ]
  					      } keys %{$large_fields}),
  					],
  		     ];
    my $response     = $self->retry(sub {
  				    my $response = $util->post($obj_uri, $payload);
  				    if(!$response->is_success()) {
  				      croak $response->status_line();
  				    }
  				    return $response;
  				  });


    my $content = $response->content();
    #########
    # Error detection here should really be status-based to be trapped by LWP
    # but we're using CGI scripts rather than mod_perl handlers so the status can't be overridden
    #
    # This is *really* bad error handling.
    #
    if($content =~ /<h\d>An\ Error\ Occurred/smix) {
      my ($errstr) = $content =~ m{<p>(Error.*?)</p>}smix;
      croak $errstr;
    }

    $self->{'read_dom'} = $util->parser->parse_string($content);
    my $root = $self->{'read_dom'}->getDocumentElement();

    for my $field ($self->fields()) {
      $self->{$field} = $root->getAttribute($field);
  #      print STDERR qq(Discovered $field == $self->{$field} in DOM\n);
    }

  #  print {*STDERR} qq(Newly assigned $obj_pk is @{[$self->$obj_pk()]}\n);
    return 1;
  }

  sub update {
    my $self       = shift;
    my $util       = $self->util();
    my ($obj_type) = (ref $self) =~ /([^:]+)$/mx;
    my $obj_pk     = $self->primary_key();
    my $obj_pk_val = $self->$obj_pk();

    if(!$obj_pk_val) {
      croak qq(Cannot update a $obj_type without an existing $obj_pk);
    }

    my $obj_uri = sprintf '%s/%s/%d', $util->base_uri(), $obj_type, $obj_pk_val;
    $DEBUG and carp qq(Posting $obj_uri using util=$util);

    my $large_fields = { map { $_ => undef } $self->large_fields() };
    my @small_fields = grep { !exists $large_fields->{$_} } $self->fields();
    my $payload      = ['Content_Type' => 'form-data',
  		      'Content'      => [
  					 'pipeline' => 1,
  					 (map { $_ => $self->{$_} || q() } grep { exists $self->{$_} } @small_fields),
  					 (map { $_ => [
  						       undef,
  						       $_,
  						       'Content_Type'   => 'binary/octet-stream',
  						       'Content_Length' => length ($self->{$_}||q()),
  						       'Content'        => $self->{$_}||q(),
  						      ]
  					      } grep { exists $self->{$_} } keys %{$large_fields}),
  					],
  		     ];
    my $response     = $self->retry(sub {
  				    my $response = $util->post($obj_uri, $payload);
  				    if(!$response->is_success()) {
  				      croak $response->status_line();
  				    }
  				    return $response;
  				  });

    my $content = $response->content();
    #########
    # Error detection here should really be status-based to be trapped by LWP
    # but we're using CGI scripts rather than mod_perl handlers so the status can't be overridden
    #
    # This is *really* bad error handling.
    #
    if($content =~ /<h\d>An\ Error\ Occurred/smix) {
      my ($errstr) = $content =~ m{<p>(Error.*?)</p>}smix;
      croak $errstr;
    }

    $self->{'read_dom'} = $util->parser->parse_string($content);
    my $root = $self->{'read_dom'}->getDocumentElement();

    for my $field ($self->fields()) {
      $self->{$field} = $root->getAttribute($field);
  #      print STDERR qq(Discovered $field == $self->{$field} in DOM\n);
    }

  #  print {*STDERR} qq(Newly assigned $obj_pk is @{[$self->$obj_pk()]}\n);
    return 1;
  }

  sub retry {
    my ($self, $cb, $max_tries, $retry_delay) = @_;

    $max_tries   ||= $MAX_RETRIES;
    $retry_delay ||= $RETRY_DELAY;

    my $retry = 0;
    my $result;

    while($retry < $max_tries) {
      $retry++;
      $DEBUG and carp "Try $retry $cb";
      eval {
        $result = $cb->();
	1;
      } or do {
        $DEBUG and carp "Try $retry gave error '$EVAL_ERROR'";
      };
      if($result) {
        last;
      }

      if($retry >= $max_tries) {
        croak "Failed $max_tries attempts. Giving up.";
      }

      $DEBUG and carp "Sleeping for $retry_delay";
      sleep $retry_delay;
    }

    $DEBUG and carp "Try $retry yielded @{[$result||'undef']}";
    return $result;
  }

}

1;
__END__

=head1 NAME

npg_qc::api::base

=head1 VERSION

$LastChangedRevision: 3504 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 new - Constructor

  my $oDerived = npg::api::<derived>->new({'util' => $oUtil,});

  my $oDerived = npg::api::<derived>->new({
    # 'key' => value parameters for the derived class
  });

=head2 fields - accessors for this table/class

  my @aFields = $oPkg->fields();
  my @aFields = npg::api::<pkg>->fields();

=head2 base_uri - returns the base uri for npg_qc

=head2 npg_uri - returns the uri for npg 

=head2 large_fields - accessors for large/lazy fields

  my @largeFields = $oPkg->large_fields();

=head2 primary_key - the first element from fields() unless overridden

  my $sPrimaryField = $oDerived->primary_key();

=head2 init - post-constructor initialisation. Called by new();

  Useful for performing additional initialisation on construction for subclasses.

  $oDerived->init();

=head2 util - util instance for this resource set

  Returns a cached object if one exists, or creates and caches a new one

  my $oUtil = $oDerived->util();

=head2 new_from_xml - create and populate an npg_qc::api::* from a XML::LibXML::Element

  my $oObj = $oDerived->new_from_xml('npg_qc::api::<package>', $oXMLElement);

=head2 new_from_json - create and populate an npg_qc::api::* from a JSON string, or a hash

  my $oObj = $oDerived->new_from_json('npg_qc::api::<package>', $json_string);
  my $oObj = $oDerived->new_from_json('npg_qc::api::<package>', $href);

=head2 hasa - makes one:one accessors on compilation

 package npg::api::<base-subclass>;
 use strict;
 use warnings;

 __PACKAGE__->hasa(['child1', 'child2', {'current_child3' => 'child3-package'}]);

=head2 hasmany - makes one:many accessors on compilation

 package npg::api::<base-subclass>;
 use strict;
 use warnings;

 __PACKAGE__->hasmany(['child1', 'child2', {'current_child3' => 'child3-package'}]);

=head2 get - overridden get accessor from Class::Accessor

Fetches XML over HTTP based on self->util->base_uri().

=head2 list - default handling for reading lists of objects

  my $oDOM = $oDerived->list();

=head2 read - default handling for reading individual objects

  my $oDOM = $oDerived->read();

=head2 create - default handling object creation

  my $oDOM = $oDerived->create();

  This creates an HTTP POST request with a multipart/form-data flavour
  and pushes object member data, for a new entity, up to the
  create_xml aspects of the service.

=head2 update - default handling object update

  my $oDOM = $oDerived->update();

  This creates an HTTP POST request with a multipart/form-data flavour
  and pushes object member data, for an existing entity ,up to the
  update_xml aspects of the service.

=head2 retry - method retries connecting to npg_qc before croaking with an error=head1 DIAGNOSTICS

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

Lingua::EN::Inflect
Class::Accessor
npg_qc::api::util
JSON

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
