#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-06-10
# Last Modified: $Date: 2008-12-12 12:07:02 +0000 (Fri, 12 Dec 2008) $
# Id:            $Id: loader.pm 3770 2008-12-12 12:07:02Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/lib/npg_qc/api/loader.pm $
#

package npg_qc::api::loader;

use strict;
use warnings;
use Class::Std;
use Carp qw(carp croak confess cluck);
use English qw{-no_match_vars};
use base qw(npg_qc::api::base);

our $VERSION = do { my ($r) = q$LastChangedRevision: 3770 $ =~ /(\d+)/mx; $r; };

{
  ## no critic
  my %xml_of       :ATTR( init_arg => 'xml', :get<xml>, :set<xml> );
  my %dbh_of       :ATTR( :get<dbh>,       :set<dbh>      );
  my %id_run_of    :ATTR( :get<id_run>,    :set<id_run>   );
  my %run_tiles_of :ATTR( :get<run_tiles>, :set<run_tiles> );
  ## use critic

  sub dbh {
    my ($self, $dbh) = @_;
    if ($dbh) {
      $self->set_dbh($dbh);
    }
    if (!$self->get_dbh()) {
      $dbh = $self->util->dbh();
      $self->set_dbh($dbh);
    }
    return $self->get_dbh();
  }

  sub run_tile {
    my ($self, $args) = @_;
    
    if (!$self->get_run_tiles()) {
      $self->set_run_tiles({});
    }

    my $run_tiles = $self->get_run_tiles();
    
    my $run_tile_ref = $args->{id_run} . q{_} . $args->{lane} . q{_} . $args->{tile} . q{_} . $args->{end};
    
    if (!$run_tiles->{$run_tile_ref}) {
    
      my $id_run_tile = $self->dbh->selectall_arrayref(
        qq{SELECT id_run_tile
           FROM run_tile
           WHERE id_run = $args->{id_run}
           AND position = $args->{lane}
           AND tile     = $args->{tile}
           AND end      = $args->{end}}, {})->[0]->[0];

      if (!$id_run_tile) {
        eval {
	  $self->dbh->do(q{INSERT INTO run_tile(id_run, position, tile, end) VALUES (?,?,?,?)}, {}, $args->{id_run},$args->{lane},$args->{tile},$args->{end});
          $id_run_tile = $self->dbh->selectall_arrayref(q{SELECT LAST_INSERT_ID()})->[0]->[0];
	  1;
	} or do {
	  $self->dbh->rollback();
	  croak 'Unable to create new run_tile';
	};
      }
      $args->{id_run_tile} = $id_run_tile;
      $run_tiles->{$run_tile_ref} = $args;

    }
    return $run_tiles->{$run_tile_ref};
  }

}
1;
__END__

=head1 NAME

npg_qc::api::loader

=head1 VERSION

$LastChangedRevision: 3770 $

=head1 SYNOPSIS

  my $oLoaderExt = npg_qc::api::loader::<ext>->new({util => $oUtil});

=head1 DESCRIPTION

Base class for loaders

=head1 SUBROUTINES/METHODS

=head2 dbh - stores/creates/returns the dbh needed to save data to the correct database

=head2 run_tile - fetches/creates/returns the unique stats for a run_tile given the 4 factors which make it unique

  my $hRunTile = $oLoaderExt->run_tile(
    {
      id_run => $id_run,
      lane   => $lane/$position,
      tile   => $tile,
      end    => $end
    });

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Class::Std
English
Carp
npg_qc::api::base

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
