#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-06-10
# Last Modified: $Date: 2008-12-12 12:07:02 +0000 (Fri, 12 Dec 2008) $
# Id:            $Id: qcal.pm 3770 2008-12-12 12:07:02Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/lib/npg_qc/api/loader/qcal.pm $
#

package npg_qc::api::loader::qcal;

use strict;
use warnings;
use Class::Std;
use Carp qw(carp croak confess cluck);
use English qw{-no_match_vars};
use base qw(npg_qc::api::loader);

our $VERSION = do { my ($r) = q$LastChangedRevision: 3770 $ =~ /(\d+)/mx; $r; };

{

  sub read_xml {
    my ($self) = @_;

    my $parser = $self->util->parser();
    my $parsed_xml = $parser->parse_string($self->get_xml());

    my $rows = $parsed_xml->getElementsByTagName('row');

    eval {
      $self->process_rows($rows);
      1;
    } or do {
      $self->dbh->disconnect();
      croak $EVAL_ERROR;
    };
    $self->dbh->commit();
    return 1;
  }

  sub process_rows {
    my ($self, $rows) = @_;

    foreach my $row (@{$rows}) {

      my $id_run     = $row->getAttribute('id_run');
      my $position   = $row->getAttribute('position');
      my $end        = $row->getAttribute('end');
      my $twenty     = $row->getAttribute('twenty');
      my $twentyfive = $row->getAttribute('twentyfive');
      my $thirty     = $row->getAttribute('thirty');

      my $query = q{INSERT into qcal(id_run,position,end,twenty,twentyfive,thirty)
                   VALUES (?,?,?,?,?,?)};

      eval {
        $self->dbh->do($query, {}, $id_run,$position,$end,$twenty,$twentyfive,$thirty);
        1;
      } or do {
        if ($EVAL_ERROR =~ /Duplicate\ entry/xms) {
          carp $EVAL_ERROR. q{ - continuing};
        } else {
          $self->dbh->rollback();
          croak $EVAL_ERROR;
        }
      };

    }
    return 1;
  }

}
1;
__END__

=head1 NAME

npg_qc::api::loader::qcal

=head1 VERSION

$LastChangedRevision: 3770 $

=head1 SYNOPSIS

  my $oQCal = npg_qc::api::loader::qcal->new({util => $oUtil, xml => $qcal_xml_string});

=head1 DESCRIPTION

Loads the XML format of the qcal files into the database directly

=head1 SUBROUTINES/METHODS

=head2 read_xml - reads the qcal_xml_string, and then saves the data out to the tables

=head2 process_rows - takes each of the rows and saves it to the signal_mean table

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Class::Std
English
Carp
npg_qc::api::base

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
