#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-06-10
# Last Modified: $Date: 2008-12-12 12:07:02 +0000 (Fri, 12 Dec 2008) $
# Id:            $Id: run_config.pm 3770 2008-12-12 12:07:02Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/lib/npg_qc/api/loader/run_config.pm $
#

package npg_qc::api::loader::run_config;

use strict;
use warnings;
use Class::Std;
use Carp qw(carp croak confess cluck);
use English qw{-no_match_vars};
use base qw(npg_qc::api::loader);

our $VERSION = do { my ($r) = q$LastChangedRevision: 3770 $ =~ /(\d+)/mx; $r; };

{
  ## no critic
  my %config_text_of :ATTR( :get<config_text>, :set<config_text> );
  ## use critic

  sub read_xml {
    my ($self) = @_;
    my $content = $self->get_xml();
    my ($id_run) = $content =~ /id_run=\"(\d+)\"/xms;
    my ($config_text) = $content =~ /config_text=\"(.*)\"\ \/>/xms;
    $self->set_id_run($id_run);
    $self->set_config_text($config_text);
    $self->save();
    return 1;
  }

  sub save {
    my ($self) = @_;
    my $dbh = $self->dbh();

    eval {
      $dbh->do(q{INSERT into run_config(id_run, config_text) VALUES (?,?)},
                {}, $self->get_id_run(), $self->get_config_text());
      $dbh->commit();
      1;
    } or do {
      if ($EVAL_ERROR =~ /Duplicate\ entry/xms) {
        carp 'Duplicate entry for run_config - continuing';
      } else {
        $dbh->disconnect();
        croak $EVAL_ERROR;
      }
    };
    return 1;
  }

}
1;
__END__

=head1 NAME

npg_qc::api::loader::run_config

=head1 VERSION

$LastChangedRevision: 3770 $

=head1 SYNOPSIS

  my $oLoaderRunConfig = npg_qc::api::loader::run_config->new({util => $oUtil, xml => $run_config_xml_string});

=head1 DESCRIPTION

Loads the XML format of the config file into the database directly

=head1 SUBROUTINES/METHODS

=head2 read_xml - processes the run_config xml string, and saves to table

=head2 save - saves the data out to the table

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Class::Std
English
Carp
npg_qc::api::base

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
