#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-06-10
# Last Modified: $Date: 2008-12-12 12:07:02 +0000 (Fri, 12 Dec 2008) $
# Id:            $Id: signal_mean.pm 3770 2008-12-12 12:07:02Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/lib/npg_qc/api/loader/signal_mean.pm $
#

package npg_qc::api::loader::signal_mean;

use strict;
use warnings;
use Class::Std;
use Carp qw(carp croak confess cluck);
use English qw{-no_match_vars};
use base qw(npg_qc::api::loader);

our $VERSION = do { my ($r) = q$LastChangedRevision: 3770 $ =~ /(\d+)/mx; $r; };

{

  sub read_xml {
    my ($self) = @_;

    my $parser = $self->util->parser();
    my $parsed_xml = $parser->parse_string($self->get_xml());

    my $run = $parsed_xml->getElementsByTagName('run')->shift();
    $self->set_id_run($run->getAttribute('id_run'));

    my $lanes_node = $run->getElementsByTagName('lanes')->shift();

    my @lanes = $lanes_node->getElementsByTagName('lane');

    eval {
      $self->process_lanes(\@lanes);
      1;
    } or do {
      $self->dbh->disconnect();
      croak $EVAL_ERROR;
    };
    $self->dbh->commit();
    return 1;
  }

  sub process_lanes {
    my ($self, $lanes) = @_;
    foreach my $lane (@{$lanes}) {
      my $position = $lane->getAttribute('position');
      my $signal_means = $lane->getElementsByTagName('signal_means')->shift();
      my @rows = $signal_means->getElementsByTagName('row');
      $self->process_rows($position, \@rows);
    }
    return 1;
  }

  sub process_rows {
    my ($self, $position, $rows) = @_;
    my $id_run = $self->get_id_run();

    foreach my $row (@{$rows}) {

      my $all_a  = $row->getAttribute('all_a');
      my $all_c  = $row->getAttribute('all_c');
      my $all_g  = $row->getAttribute('all_g');
      my $all_t  = $row->getAttribute('all_t');
      my $base_a = $row->getAttribute('base_a');
      my $base_c = $row->getAttribute('base_c');
      my $base_g = $row->getAttribute('base_g');
      my $base_t = $row->getAttribute('base_t');
      my $call_a = $row->getAttribute('call_a');
      my $call_c = $row->getAttribute('call_c');
      my $call_g = $row->getAttribute('call_g');
      my $call_t = $row->getAttribute('call_t');
      my $cycle  = $row->getAttribute('cycle');

      my $query = q{INSERT into signal_mean(id_run,position,cycle,all_a,all_c,all_g,all_t,base_a,base_c,base_g,base_t,call_a,call_c,call_g,call_t)
                   VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)};

      eval {
        $self->dbh->do($query, {}, $id_run,$position,$cycle,$all_a,$all_c,$all_g,$all_t,$base_a,$base_c,$base_g,$base_t,$call_a,$call_c,$call_g,$call_t);
        1;
      } or do {
        if ($EVAL_ERROR =~ /Duplicate\ entry/xms) {
          carp $EVAL_ERROR. q{ - continuing};
        } else {
          $self->dbh->rollback();
          croak $EVAL_ERROR;
        }
      };

    }
    $self->dbh->commit();
    return 1;
  }

}
1;
__END__

=head1 NAME

npg_qc::api::loader::signal_mean

=head1 VERSION

$LastChangedRevision: 3770 $

=head1 SYNOPSIS

  my $oLoaderSignalMean = npg_qc::api::loader::signal_mean->new({util => $oUtil, xml => $signal_mean_xml_string});

=head1 DESCRIPTION

Loads the XML format of the signal mean file into the database directly

=head1 SUBROUTINES/METHODS

=head2 read_xml - reads the signal_mean_xml_string, and then saves the data out to the tables

=head2 process_lanes - takes the array of lanes and deals with each lane in turn

=head2 process_rows - takes each of the rows and saves it to the signal_mean table

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Class::Std
English
Carp
npg_qc::api::base

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
