#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-06-10
# Last Modified: $Date: 2008-12-12 12:07:02 +0000 (Fri, 12 Dec 2008) $
# Id:            $Id: tile_score.pm 3770 2008-12-12 12:07:02Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/lib/npg_qc/api/loader/tile_score.pm $
#

package npg_qc::api::loader::tile_score;

use strict;
use warnings;
use Class::Std;
use Carp qw(carp croak confess cluck);
use English qw{-no_match_vars};
use base qw(npg_qc::api::loader);

our $VERSION = do { my ($r) = q$LastChangedRevision: 3770 $ =~ /(\d+)/mx; $r; };

{

  sub read_xml {
    my ($self) = @_;

    my $parser = $self->util->parser();
    my $parsed_xml = $parser->parse_string($self->get_xml());

    my @run_tiles = $parsed_xml->getElementsByTagName('run_tile');

    foreach my $run_tile (@run_tiles) {

      my $id_run = $run_tile->getAttribute('id_run');
      my $lane   = $run_tile->getAttribute('position');
      my $tile   = $run_tile->getAttribute('tile');
      my $end    = $run_tile->getAttribute('end');

      my $args = {
        id_run => $id_run,
	lane   => $lane,
	tile   => $tile,
	end    => $end
      };
      
      my $id_run_tile = $self->run_tile($args)->{id_run_tile};
      eval {
        $self->tile_score($id_run_tile, $run_tile);
        $self->cumulative_errors_by_cycle($id_run_tile, $run_tile);
        $self->log_likelihood($id_run_tile, $run_tile);
        $self->information_content_by_cycle($id_run_tile, $run_tile);
        $self->error_rate_relative_reference_cycle_nucleotide($id_run_tile, $run_tile);
        $self->errors_by_cycle($id_run_tile, $run_tile);
        $self->error_rate_reference_including_blanks($id_run_tile, $run_tile);
        $self->error_rate_reference_no_blanks($id_run_tile, $run_tile);
        $self->error_rate_relative_sequence_base($id_run_tile, $run_tile);
        $self->errors_by_nucleotide($id_run_tile, $run_tile);
        $self->errors_by_cycle_and_nucleotide($id_run_tile, $run_tile);
        $self->most_common_word($id_run_tile, $run_tile);
        $self->most_common_blank_pattern($id_run_tile, $run_tile);
        1;
      } or do {
        $self->dbh->disconnect();
        croak $EVAL_ERROR;
      };
      $self->dbh->commit();
    }

    return 1;
  }

  sub most_common_word {
    my ($self, $id_run_tile, $run_tile) = @_;
    my $table = $run_tile->getElementsByTagName('most_common_word')->shift;
    my @rows = $table->getElementsByTagName('row');

    my $query = q{INSERT INTO most_common_word(id_run_tile, rank, occurrence, word, number_of_sequences, rescore)
                VALUES (?,?,?,?,?,?)};

    my $query_select = q{SELECT id_most_common_word
                         FROM most_common_word
                         WHERE id_run_tile = ?
                         AND rank = ?
                         AND occurrence = ?
                         AND word = ?
                         AND number_of_sequences = ?
                         AND rescore = ?};

    foreach my $row (@rows) {
      my $rank                = $row->getAttribute('rank');
      my $occurrence          = $row->getAttribute('occurrence');
      my $word                = $row->getAttribute('word');
      my $number_of_sequences = $row->getAttribute('number_of_sequences');
      my $rescore             = $row->getAttribute('rescore');

      eval {
        my $id_most_common_word = $self->dbh->selectall_arrayref($query_select, {}, $id_run_tile, $rank, $occurrence, $word, $number_of_sequences, $rescore)->[0]->[0];
	if (!$id_most_common_word) {
	  $self->dbh->do($query, {}, $id_run_tile, $rank, $occurrence, $word, $number_of_sequences, $rescore);
        }
	1;
      } or do {
	if ($EVAL_ERROR =~ /Duplicate\ entry/xms) {
	 carp $EVAL_ERROR. q{ - continuing};
	} else {
	 $self->dbh->rollback();
	 croak $EVAL_ERROR;
	}
      };
     
    }
    $self->dbh->commit();
    return 1;
  }

  sub most_common_blank_pattern {
    my ($self, $id_run_tile, $run_tile) = @_;
    my $table = $run_tile->getElementsByTagName('most_common_blank_pattern')->shift;
    my @rows = $table->getElementsByTagName('row');

    my $query = q{INSERT INTO most_common_blank_pattern(id_run_tile, rank, occurrence, pattern, number_of_sequences, rescore) VALUES (?,?,?,?,?,?)};

    my $query_select = q{SELECT id_most_common_blank_pattern
                         FROM most_common_blank_pattern
                         WHERE id_run_tile = ?
                         AND rank = ?
                         AND occurrence = ?
                         AND pattern = ?
                         AND number_of_sequences = ?
                         AND rescore = ?};

    foreach my $row (@rows) {
      my $rank                = $row->getAttribute('rank');
      my $occurrence          = $row->getAttribute('occurrence');
      my $pattern             = $row->getAttribute('pattern');
      my $number_of_sequences = $row->getAttribute('number_of_sequences');
      my $rescore             = $row->getAttribute('rescore');

      eval {
        my $id_most_common_blank_pattern = $self->dbh->selectall_arrayref($query_select, {}, $id_run_tile, $rank, $occurrence, $pattern, $number_of_sequences, $rescore)->[0]->[0];
	if (!$id_most_common_blank_pattern) {
          $self->dbh->do($query, {}, $id_run_tile, $rank, $occurrence, $pattern, $number_of_sequences, $rescore);
        }
	1;
      } or do {
	if ($EVAL_ERROR =~ /Duplicate\ entry/xms) {
	 carp $EVAL_ERROR. q{ - continuing};
	} else {
	 $self->dbh->rollback();
	 croak $EVAL_ERROR;
	}
      };
     
    }
    $self->dbh->commit();
    return 1;
  }

  sub errors_by_cycle_and_nucleotide {
    my ($self, $id_run_tile, $run_tile) = @_;
    my $table = $run_tile->getElementsByTagName('errors_by_cycle_and_nucleotide')->shift;
    my @rows = $table->getElementsByTagName('row');

    my $query = q{INSERT INTO errors_by_cycle_and_nucleotide(id_run_tile, cycle, read_as, really_a, really_c, really_g, really_t, rescore) VALUES (?,?,?,?,?,?,?,?)};

    foreach my $row (@rows) {
      my $cycle    = $row->getAttribute('cycle');
      my $read_as  = $row->getAttribute('read_as');
      my $really_a = $row->getAttribute('really_a');
      my $really_c = $row->getAttribute('really_c');
      my $really_g = $row->getAttribute('really_g');
      my $really_t = $row->getAttribute('really_t');
      my $rescore  = $row->getAttribute('rescore');

      eval {
	$self->dbh->do($query, {}, $id_run_tile, $cycle, $read_as, $really_a, $really_c, $really_g, $really_t, $rescore);
	1;
      } or do {
	if ($EVAL_ERROR =~ /Duplicate\ entry/xms) {
	 carp $EVAL_ERROR. q{ - continuing};
	} else {
	 $self->dbh->rollback();
	 croak $EVAL_ERROR;
	}
      };
     
    }
    $self->dbh->commit();
    return 1;
  }

  sub errors_by_nucleotide {
    my ($self, $id_run_tile, $run_tile) = @_;
    my $table = $run_tile->getElementsByTagName('errors_by_nucleotide')->shift;
    my @rows = $table->getElementsByTagName('row');

    my $query = q{INSERT INTO errors_by_nucleotide(id_run_tile, read_as, really_a, really_c, really_g, really_t, rescore)
    VALUES (?,?,?,?,?,?,?)};

    foreach my $row (@rows) {
      my $read_as  = $row->getAttribute('read_as');
      my $really_a = $row->getAttribute('really_a');
      my $really_c = $row->getAttribute('really_c');
      my $really_g = $row->getAttribute('really_g');
      my $really_t = $row->getAttribute('really_t');
      my $rescore  = $row->getAttribute('rescore');

      eval {
	$self->dbh->do($query, {}, $id_run_tile, $read_as, $really_a, $really_c, $really_g, $really_t, $rescore);
	1;
      } or do {
	if ($EVAL_ERROR =~ /Duplicate\ entry/xms) {
	 carp $EVAL_ERROR. q{ - continuing};
	} else {
	 $self->dbh->rollback();
	 croak $EVAL_ERROR;
	}
      };
     
    }
    $self->dbh->commit();
    return 1;
  }
  sub error_rate_relative_sequence_base {
    my ($self, $id_run_tile, $run_tile) = @_;
    my $table = $run_tile->getElementsByTagName('error_rate_relative_sequence_base')->shift;
    my @rows = $table->getElementsByTagName('row');

    my $query = q{INSERT INTO error_rate_relative_sequence_base(id_run_tile, read_as, really_a, really_c, really_g, really_t, rescore)
    VALUES (?,?,?,?,?,?,?)};

    foreach my $row (@rows) {
      my $read_as  = $row->getAttribute('read_as');
      my $really_a = $row->getAttribute('really_a');
      my $really_c = $row->getAttribute('really_c');
      my $really_g = $row->getAttribute('really_g');
      my $really_t = $row->getAttribute('really_t');
      my $rescore  = $row->getAttribute('rescore');

      eval {
	$self->dbh->do($query, {}, $id_run_tile, $read_as, $really_a, $really_c, $really_g, $really_t, $rescore);
	1;
      } or do {
	if ($EVAL_ERROR =~ /Duplicate\ entry/xms) {
	 carp $EVAL_ERROR. q{ - continuing};
	} else {
	 $self->dbh->rollback();
	 croak $EVAL_ERROR;
	}
      };
     
    }
    $self->dbh->commit();
    return 1;
  }

  sub error_rate_reference_including_blanks {
    my ($self, $id_run_tile, $run_tile) = @_;
    my $table = $run_tile->getElementsByTagName('error_rate_reference_including_blanks')->shift;
    my @rows = $table->getElementsByTagName('row');

    my $query = q{INSERT INTO error_rate_reference_including_blanks(id_run_tile, really, read_as_blank, read_as_a, read_as_c, read_as_g, read_as_t, rescore)
    VALUES (?,?,?,?,?,?,?,?)};

    foreach my $row (@rows) {
      my $really        = $row->getAttribute('really');
      my $read_as_blank = $row->getAttribute('read_as_blank');
      my $read_as_a     = $row->getAttribute('read_as_a');
      my $read_as_c     = $row->getAttribute('read_as_c');
      my $read_as_g     = $row->getAttribute('read_as_g');
      my $read_as_t     = $row->getAttribute('read_as_t');
      my $rescore       = $row->getAttribute('rescore');

      eval {
	$self->dbh->do($query, {}, $id_run_tile, $really, $read_as_blank, $read_as_a, $read_as_c, $read_as_g, $read_as_t, $rescore);
	1;
      } or do {
	if ($EVAL_ERROR =~ /Duplicate\ entry/xms) {
	 carp $EVAL_ERROR. q{ - continuing};
	} else {
	 $self->dbh->rollback();
	 croak $EVAL_ERROR;
	}
      };
     
    }
    $self->dbh->commit();
    return 1;
  }

  sub error_rate_reference_no_blanks {
    my ($self, $id_run_tile, $run_tile) = @_;
    my $table = $run_tile->getElementsByTagName('error_rate_reference_no_blanks')->shift;
    my @rows = $table->getElementsByTagName('row');

    my $query = q{INSERT INTO error_rate_reference_no_blanks(id_run_tile, really, read_as_a, read_as_c, read_as_g, read_as_t, rescore)
    VALUES (?,?,?,?,?,?,?)};

    foreach my $row (@rows) {
      my $really        = $row->getAttribute('really');
      my $read_as_a     = $row->getAttribute('read_as_a');
      my $read_as_c     = $row->getAttribute('read_as_c');
      my $read_as_g     = $row->getAttribute('read_as_g');
      my $read_as_t     = $row->getAttribute('read_as_t');
      my $rescore       = $row->getAttribute('rescore');

      eval {
	$self->dbh->do($query, {}, $id_run_tile, $really, $read_as_a, $read_as_c, $read_as_g, $read_as_t, $rescore);
	1;
      } or do {
	if ($EVAL_ERROR =~ /Duplicate\ entry/xms) {
	 carp $EVAL_ERROR. q{ - continuing};
	} else {
	 $self->dbh->rollback();
	 croak $EVAL_ERROR;
	}
      };
     
    }
    $self->dbh->commit();
    return 1;
  }

  sub tile_score {
    my ($self, $id_run_tile, $run_tile) = @_;
    my $tile_score = $run_tile->getElementsByTagName('tile_score')->shift;

    my $base_count = $tile_score->getAttribute('base_count');
    my $error_count = $tile_score->getAttribute('error_count');
    my $blank_count = $tile_score->getAttribute('blank_count');
    my $unique_alignments = $tile_score->getAttribute('unique_alignments');
    my $ua_total_score = $tile_score->getAttribute('ua_total_score');
    my $cycles = $tile_score->getAttribute('cycles');
    my $rescore = $tile_score->getAttribute('rescore');
    my $score_version = $tile_score->getAttribute('score_version');
    my $score_date_run = $tile_score->getAttribute('score_date_run');
    my $phagealign_version = $tile_score->getAttribute('phagealign_version');
    my $phagealign_date_run = $tile_score->getAttribute('phagealign_date_run');
    my $max_blanks = $tile_score->getAttribute('max_blanks');
    my $seq_length = $tile_score->getAttribute('seq_length');
    my $genome_file = $tile_score->getAttribute('genome_file');
    my $bases_used = $tile_score->getAttribute('bases_used');
    my $qualityfilter_version = $tile_score->getAttribute('qualityfilter_version');
    my $qualityfilter_date_run = $tile_score->getAttribute('qualityfilter_date_run');
    my $filter_criterion = $tile_score->getAttribute('filter_criterion');

    my $query = q{INSERT INTO tile_score(id_run_tile,base_count,error_count,blank_count,unique_alignments,ua_total_score,cycles,rescore,score_version,score_date_run,phagealign_version,phagealign_date_run,max_blanks,seq_length,genome_file,bases_used,qualityfilter_version,qualityfilter_date_run,filter_criterion)
    VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)};
    eval {
      $self->dbh->do($query, {}, $id_run_tile,$base_count,$error_count,$blank_count,$unique_alignments,$ua_total_score,$cycles,$rescore,$score_version,$score_date_run,$phagealign_version,$phagealign_date_run,$max_blanks,$seq_length,$genome_file,$bases_used,$qualityfilter_version,$qualityfilter_date_run,$filter_criterion);
      1;
    } or do {
      if ($EVAL_ERROR =~ /Duplicate\ entry/xms) {
    	carp $EVAL_ERROR. q{ - continuing};
      } else {
    	$self->dbh->rollback();
    	croak $EVAL_ERROR;
      }
    };

    $self->dbh->commit();
    return 1;
  }

  sub cumulative_errors_by_cycle {
    my ($self, $id_run_tile, $run_tile) = @_;
    my $cebc = $run_tile->getElementsByTagName('cumulative_errors_by_cycle')->shift;
    my @rows = $cebc->getElementsByTagName('row');

    my $query = q{INSERT INTO cumulative_errors_by_cycle(id_run_tile, cycle, one, two, three, four, five, rescore)
                  VALUES (?,?,?,?,?,?,?,?)};
    foreach my $row (@rows) {
      my $cycle   = $row->getAttribute('cycle');
      my $one     = $row->getAttribute('one');
      my $two     = $row->getAttribute('two');
      my $three   = $row->getAttribute('three');
      my $four    = $row->getAttribute('four');
      my $five    = $row->getAttribute('five');
      my $rescore = $row->getAttribute('rescore');
      eval {
        $self->dbh->do($query, {}, $id_run_tile,$cycle, $one, $two, $three, $four, $five, $rescore);
        1;
      } or do {
        if ($EVAL_ERROR =~ /Duplicate\ entry/xms) {
    	  carp $EVAL_ERROR. q{ - continuing};
        } else {
    	  $self->dbh->rollback();
    	  croak $EVAL_ERROR;
        }
      };
      
    }

    $self->dbh->commit();
    return 1;
  }

  sub log_likelihood {
    my ($self, $id_run_tile, $run_tile) = @_;
    my $ll = $run_tile->getElementsByTagName('log_likelihood')->shift;
    my @rows = $ll->getElementsByTagName('row');

    my $query = q{INSERT INTO log_likelihood(id_run_tile, cycle, read_as, really_a, really_c, really_g, really_t, rescore)
                  VALUES (?,?,?,?,?,?,?,?)};

    foreach my $row (@rows) {
      my $cycle    = $row->getAttribute('cycle');
      my $read_as  = $row->getAttribute('read_as');
      my $really_a = $row->getAttribute('really_a');
      my $really_c = $row->getAttribute('really_c');
      my $really_g = $row->getAttribute('really_g');
      my $really_t = $row->getAttribute('really_t');
      my $rescore  = $row->getAttribute('rescore');

      eval {
        $self->dbh->do($query, {}, $id_run_tile, $cycle, $read_as, $really_a, $really_c, $really_g, $really_t, $rescore);
        1;
      } or do {
        if ($EVAL_ERROR =~ /Duplicate\ entry/xms) {
    	  carp $EVAL_ERROR. q{ - continuing};
        } else {
    	  $self->dbh->rollback();
    	  croak $EVAL_ERROR;
        }
      };
      
    }
    $self->dbh->commit();
    return 1;
  }
  sub information_content_by_cycle {
    my ($self, $id_run_tile, $run_tile) = @_;
    my $table = $run_tile->getElementsByTagName('information_content_by_cycle')->shift;
    my @rows = $table->getElementsByTagName('row');

    my $query = q{INSERT INTO information_content_by_cycle(id_run_tile, cycle, equiv_info, align, total, rescore)
                  VALUES (?,?,?,?,?,?)};

    foreach my $row (@rows) {
      my $cycle      = $row->getAttribute('cycle');
      my $equiv_info = $row->getAttribute('equiv_info');
      my $align      = $row->getAttribute('align');
      my $total      = $row->getAttribute('total');
      my $rescore    = $row->getAttribute('rescore');

      eval {
        $self->dbh->do($query, {}, $id_run_tile, $cycle, $equiv_info, $align, $total, $rescore);
        1;
      } or do {
        if ($EVAL_ERROR =~ /Duplicate\ entry/xms) {
    	  carp $EVAL_ERROR. q{ - continuing};
        } else {
    	  $self->dbh->rollback();
    	  croak $EVAL_ERROR;
        }
      };
      
    }
    $self->dbh->commit();
    return 1;
  }

  sub error_rate_relative_reference_cycle_nucleotide {
    my ($self, $id_run_tile, $run_tile) = @_;
    my $table = $run_tile->getElementsByTagName('error_rate_relative_reference_cycle_nucleotide')->shift;
    my @rows = $table->getElementsByTagName('row');

    my $query = q{INSERT INTO error_rate_relative_reference_cycle_nucleotide(id_run_tile, cycle, read_as, really_a, really_c, really_g, really_t, rescore) VALUES (?,?,?,?,?,?,?,?)};

    foreach my $row (@rows) {
      my $cycle    = $row->getAttribute('cycle');
      my $read_as  = $row->getAttribute('read_as');
      my $really_a = $row->getAttribute('really_a');
      my $really_c = $row->getAttribute('really_c');
      my $really_g = $row->getAttribute('really_g');
      my $really_t = $row->getAttribute('really_t');
      my $rescore  = $row->getAttribute('rescore');

      eval {
	$self->dbh->do($query, {}, $id_run_tile, $cycle, $read_as, $really_a, $really_c, $really_g, $really_t, $rescore);
	1;
      } or do {
	if ($EVAL_ERROR =~ /Duplicate\ entry/xms) {
	 carp $EVAL_ERROR. q{ - continuing};
	} else {
	 $self->dbh->rollback();
	 croak $EVAL_ERROR;
	}
      };
     
    }
    $self->dbh->commit();
    return 1;
  }

  sub errors_by_cycle {
    my ($self, $id_run_tile, $run_tile) = @_;
    my $table = $run_tile->getElementsByTagName('errors_by_cycle')->shift;
    my @rows = $table->getElementsByTagName('row');

    my $query = q{INSERT INTO errors_by_cycle(id_run_tile, cycle, error_percentage, blank_percentage, rescore)
                  VALUES (?,?,?,?,?)};

    foreach my $row (@rows) {
      my $cycle            = $row->getAttribute('cycle');
      my $error_percentage = $row->getAttribute('error_percentage');
      my $blank_percentage = $row->getAttribute('blank_percentage');
      my $rescore          = $row->getAttribute('rescore');

      eval {
	$self->dbh->do($query, {}, $id_run_tile, $cycle, $error_percentage, $blank_percentage, $rescore);
	1;
      } or do {
	if ($EVAL_ERROR =~ /Duplicate\ entry/xms) {
	 carp $EVAL_ERROR. q{ - continuing};
	} else {
	 $self->dbh->rollback();
	 croak $EVAL_ERROR;
	}
      };
     
    }
    $self->dbh->commit();
    return 1;
  }



}
1;
__END__

=head1 NAME

npg_qc::api::loader::tile_score

=head1 VERSION

$LastChangedRevision: 3770 $

=head1 SYNOPSIS

  my $oQCal = npg_qc::api::loader::tile_score->new({util => $oUtil, xml => $tile_score_xml_string});

=head1 DESCRIPTION

Loads the XML format of the qcal files into the database directly

=head1 SUBROUTINES/METHODS

=head2 read_xml - reads the tile_score_xml_string, and then saves the data out to the tables

=head2 tile_score
=head2 cumulative_errors_by_cycle
=head2 log_likelihood
=head2 information_content_by_cycle
=head2 error_rate_relative_reference_cycle_nucleotide
=head2 errors_by_cycle
=head2 error_rate_reference_including_blanks
=head2 error_rate_reference_no_blanks
=head2 error_rate_relative_sequence_base
=head2 errors_by_nucleotide
=head2 errors_by_cycle_and_nucleotide
=head2 most_common_word
=head2 most_common_blank_pattern

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Class::Std
English
Carp
npg_qc::api::base

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
