#########
# Author:        gq1
# Maintainer:    $Author: gq1 $
# Created:       2008-09-03
# Last Modified: $Date: 2008-09-12 15:30:50 +0100 (Fri, 12 Sep 2008) $
# Id:            $Id: runlog_z3.pm 3022 2008-09-12 14:30:50Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/lib/npg_qc/api/process/runlog_z3.pm $
#

package npg_qc::api::process::runlog_z3;

use strict;
use warnings;
use English qw(-no_match_vars);
use Carp;
use Class::Std;
use base qw(npg_qc::api::process);
use Readonly;
use npg_qc::model::move_z;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3022 $ =~ /(\d+)/mx; $r; };

Readonly our $TILES_PER_XML => 90;

{
  ## no critic
  my %xml_list_movez_of     :ATTR( :get<xml_list_movez>, :set<xml_list_movez>);
  my %tile_key_of           :ATTR( :get<tile_key>, :set<tile_key>);
  my %tile_list_of          :ATTR( :get<tile_list>, :set<tile_list>);
  my %current_cycle_num_of  :ATTR( :get<current_cycle_num>, :set<current_cycle_num>);
  my %num_tiles_run_of      :ATTR( :get<num_tiles_run>, :set<num_tiles_run>);
  ## use critic

  sub extract_movez{
    my ($self, $runlog_file_url) = @_;
        
    $self->process_id_run($runlog_file_url);

    open my $logfile, '<', $runlog_file_url or croak "can't open $runlog_file_url\n";
    $self->read_log($logfile);
    close $logfile or croak "can't close the file handler $runlog_file_url\n";

    return $self->get_xml_list_movez();
  }

  sub read_log{
    my ($self, $logfile)= @_;

    $self->set_xml_list_movez([]);
    $self->set_tile_list({});
    
    my $line_count =0;

    my $start_cycle = 0;
    my $end_cycle = 0;

    my $cycle_tag_open = 0;
    my $cycle_tag_close = 1;

    my $tile_tag_open = 0;
    my $tile_tag_close = 1;


    while (my $line = <$logfile>) {
      $line_count++;

      #check open cycle or close cycle
      $start_cycle = $self->start_tag($line, 'IMAGE_CYCLE');
      $end_cycle = $self->end_tag($line, 'IMAGE_CYCLE');
      if($start_cycle && $end_cycle){
        #carp "ignore one line IMAGE_CYCLE tag at line $line_count: $line";
      }elsif ($start_cycle){
        $tile_tag_open = 0;
        $tile_tag_close = 1;

        if(!$cycle_tag_close) {
         # carp "cycle tag not close before line $line_count: $line";
          $cycle_tag_close = 1;
        }

        my ($current_cycle_num) =  $line =~ /<IMAGE_CYCLE.+Cycle="(\d+)"/mx;
        if(defined $current_cycle_num){
          $self->set_current_cycle_num($current_cycle_num);
        }#else{
          #carp "cycle number missing at line $line_count: $line";
        #}

        #print $current_cycle_num, ":$line";
        $cycle_tag_open = 1;
        $cycle_tag_close = 0;

      }elsif ($end_cycle){
        if(!$cycle_tag_open){
          #carp "cycle tag not open before line $line_count: $line";
        }
       # print $current_cycle_num, "-$line";
        $cycle_tag_close = 1;
        $cycle_tag_open = 0;
      }

      #if within one cycle
      if($cycle_tag_open && !$cycle_tag_close){

       #check open tile or close tile
        my $start_tile_tag = $self->start_tag($line, 'TILE');
        my $end_tile_tag = $self->end_tag($line, 'TILE');

        if($start_tile_tag && !$end_tile_tag){

          if(!$tile_tag_close){
            #carp "title tag not close before line $line_count: $line";
            $tile_tag_close = 1;
          }
          $self->set_tile_key(q{});
          $tile_tag_open = 1;
          $tile_tag_close = 0;

        }elsif($end_tile_tag && !$start_tile_tag){
          $tile_tag_close = 1;
          $tile_tag_open = 0;
          $self->set_tile_key(q{});
        }

        if($tile_tag_open && !$tile_tag_close){
           $self->parse_movez_line($line, $line_count);
           $self->parse_tile_line($line);
        }
      }
    }

    $self->covert_double_hash2xml_list($self->get_tile_list());
    return;
  }

  sub covert_double_hash2xml_list{
    my ($self, $tilelist) = @_;

    $self->set_num_tiles_run (scalar keys %{$tilelist});

    my @xml_list_movez;

    my $count = 0;
    my $xml_tile;

    foreach my $tile_tag (sort keys %{$tilelist} ) {
      if($count == 0){
        $xml_tile = qq{<?xml version="1.0" encoding="utf-8"?>\n<root>\n<dataset>z_log</dataset>\n};
        $xml_tile .= "<tiles>\n";
      }
      $count++;
      $xml_tile .= "$tile_tag";
      
      my $cycle_list_per_tile = $tilelist->{$tile_tag};
      my $move_z_list = q{}; #empty string
      my $sum_newz = 0;
      my $num_newz = 0;
      foreach my $cycle (sort{ $a <=> $b } keys %{$cycle_list_per_tile }){
        my $move_z = $cycle_list_per_tile->{$cycle};
        my $move_z_values  = sprintf q[ %s="%s"],q{<move_z cycle}, $move_z->cycle();
    	     $move_z_values .= sprintf q[ %s="%s"],q{ start}, $move_z->start();
    	     $move_z_values .= sprintf q[ %s="%s"],q{ stop}, $move_z->stop();
    	     $move_z_values .= sprintf q[ %s="%s"],q{ currentz},$move_z->currentz() ;
    	     $move_z_values .= sprintf q[ %s="%s"],q{ targetz}, $move_z->targetz();
    	     $move_z_values .= sprintf q[ %s="%s"],q{ newz}, $move_z->newz();
    	     $move_z_values .= sprintf q[ %s="%s"],q{ move},$move_z->move() ;
    	     $move_z_values .= qq{ />\n};
           #print $move_z_values, "\n";
           if($cycle != 1){
             $sum_newz += $move_z->newz();
             $num_newz ++;
           }
           $move_z_list .= $move_z_values;
      }
      my $avg_newz;
      
      if($num_newz >0){
      
        $avg_newz = $sum_newz / $num_newz;
      
        $xml_tile .= sprintf q[ %s="%.2f"],q{ avg_newz}, $avg_newz;
      }
      $xml_tile .= qq{>\n};
      $xml_tile .= $move_z_list;
      #$xml_tile .= $tilelist->{$tile_tag};
      $xml_tile .= "</tile>\n";
      if($count == $TILES_PER_XML){
        $xml_tile .="</tiles>\n";
        $xml_tile .= "</root>\n";
        $count = 0;
        push @xml_list_movez, $xml_tile;
      }
    }

    if($count > 0 && $count < $TILES_PER_XML){
      $xml_tile .="</tiles>\n";
      $xml_tile .= "</root>\n";
      $count = 0;
      push @xml_list_movez, $xml_tile;
    }

    $self->set_xml_list_movez(\@xml_list_movez);
    return;
  }


  sub parse_tile_line{
    my ($self, $line) = @_;
    
    if($line =~ /<TILE/mx){
             #<TILE start="307999047" Lane="0" Column="0" Row="16" TileIndex="16" NumImages="96 (wait)" ImageQueue="0">
             my ($current_lane) = $line =~ /Lane="(\d+)"/smxi;
             my ($current_col) = $line =~ /Column="(\d+)"/smxi;
             my ($current_row) = $line =~ /Row="(\d+)"/smxi;
             my ($current_tile) = $line =~ /TileIndex="(\d+)"/smxi;

             if(defined $current_lane && defined $current_col && defined $current_row && defined $current_tile){
               #increase 1, to make they count from 1
               my $current_lane_from1 = $current_lane + 1;
               my $current_tile_from1 = $current_tile + 1;
             
               my $tile_key = q{}; #empty string
               $tile_key .= sprintf q[%s="%s"], q{<tile id_run}, $self->get_id_run();
               $tile_key .= qq{ row="$current_row"}
      		             .qq{ col="$current_col"}
    	  		             .qq{ position="$current_lane_from1"}
    	  		             .qq{ tile="$current_tile_from1"};
               $tile_key.= sprintf q[ %s="%s"], q{end}, q{1};
               #$tile_key.= qq{>\n};
               $self->set_tile_key($tile_key);
               #print $tile_key, " tile\n";
             }
             #else{
               #carp "missing data at line: $line";
             #}
 
    }
    return;
  }
  sub parse_movez_line{
    my ($self, $line, $line_count) = @_;

    if($line =~ /<MOVE_Z/mx){
      my $move_z_values = q{}; #empty string

      # <MOVE_Z start="307974797" stop="307974797" CurrentZ="16970" TargetZ="16800" NewZ="16970" MoveZ="N"
      my ($start) = $line =~ /start="(\d+)"/smxi;
      my ($stop) = $line =~ /stop="(\d+)"/smxi;
      my ($currentz) = $line =~ /CurrentZ="(-?\d+)"/smxi;
      my ($targetz) = $line =~ /TargetZ="(-?\d+)"/smxi;
      my ($newz) = $line =~ /NewZ="(\-?\d+)"/smxi;
      my ($move) = $line =~ /MoveZ="(\w)"/smxi;
      if(  defined $start
        && defined $stop
        && defined $currentz
        && defined $targetz
        && defined $newz
        && defined $move ){

        my $move_value = 0;
        if($move eq 'Y'){
        	     $move_value = 1;
        }
        my $current_move_z = undef;
        $current_move_z = npg_qc::model::move_z->new();

        $current_move_z->start($start);
        $current_move_z->stop($stop);
        $current_move_z->currentz($currentz);
        $current_move_z->targetz($targetz);
        $current_move_z->newz($newz);
        $current_move_z->move($move_value);
        $current_move_z->cycle($self->get_current_cycle_num());
        #print $current_move_z->start(), " move_z start\n";
        #$move_z_values .= sprintf q[ %s="%s"],q{<move_z cycle}, $self->get_current_cycle_num();
    	  #$move_z_values .= qq{ start="$start"}
    		#                .qq{ stop="$stop"}
    		#                .qq{ currentz="$currentz"}
    		#                .qq{ targetz="$targetz"}
    		#               .qq{ newz="$newz"}
    		#              .qq{ move="$move_value"}
    	   #             .= qq{ />\n};
        #print $move_z_values, "\n";
        
        my $current_tile_key = $self->get_tile_key();

        if($current_tile_key ne q{}){
          my $old_move_z = $self->get_tile_list()->{$current_tile_key}->{$self->get_current_cycle_num()};
          if (($old_move_z && $old_move_z->start()<$current_move_z->start()) || !$old_move_z){
    	        $self->get_tile_list()->{$current_tile_key}->{$self->get_current_cycle_num()} = $current_move_z;

          }
        }
      }
      #else{
         #carp "data missing in line $line_count: $line";
      #}

    }
    return;
  }
  sub start_tag{
    my ($self, $line, $tag_name) = @_;

    return $line =~ /<$tag_name/mx;
  }

  sub end_tag{
    my ($self, $line, $tag_name) = @_;

    return $line =~ /<\/$tag_name>/mx || $line =~ /<$tag_name.+\/>/mx;
  }

}
1;

__END__

=head1 NAME

npg_qc::api::process::runlog_z3

=head1 VERSION

$LastChangedRevision: 3022 $

=head1 SYNOPSIS

  my $oRunlog_z = npg_qc::api::process::runlog_z->new({util => $util});

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2       covert_double_hash2xml_list - write a hash (titetag=>cycle=>move_z) into a list of xml string

=head2       end_tag  - check one line is a end tag of a given name

=head2       extract_movez - main methods return a list of xml strings/

=head2       parse_movez_line - check one line is a move line or not, if so get the values

=head2       parse_tile_line - check one line is a tile line or not, if so get the value

=head2       read_log  - read the runlog file line by line

=head2       start_tag - check one line is a start tag of a given name


=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
English
Carp

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Guoying Qi, E<lt>gq1@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Guoying Qi

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut

