#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-06-19
# Last Modified: $Date: 2008-12-01 11:14:26 +0000 (Mon, 01 Dec 2008) $
# Id:            $Id: lane.pm 3636 2008-12-01 11:14:26Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/lib/npg_qc/api/run/lane.pm $
#
package npg_qc::api::run::lane;
use strict;
use warnings;
use Class::Std;
use Carp;
use base qw(npg_qc::api::run);
use Readonly;
use JSON;
use npg_qc::api::run::lane::tile;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3636 $ =~ /(\d+)/mx; $r; };

{
  ##no critic
  my %lane_of     :ATTR( :get<lane>,     :set<lane>     );
  my %tiles_of    :ATTR( :get<tiles>,    :set<tiles>    );
  my %read_end_of :ATTR( :get<read_end>, :set<read_end> );
  ##use critic

  sub fields {
    return qw(
              lane
            );
  }
  
  sub create_tiles {
    my ($self, $tiles_array) = @_;
    $self->set_tiles([]);
    foreach my $tile (@{$tiles_array}) {
      $tile->{avg_newz} = $tile->{tile_qc_data}->{avg_newz};
      my $tile_obj = $self->new_from_json('npg_qc::api::run::lane::tile', $tile);
      $tile_obj->id_run($self->id_run());
      $tile_obj->set_lane($self->get_lane());
      $tile_obj->set_read_end($self->get_read_end());
      push @{$self->get_tiles()}, $tile_obj;
    }
    return 1;
  }
}
1;
__END__
=head1 NAME

npg_qc::api::run::lane

=head1 VERSION

$LastChangedRevision: 3636 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 new - Constructor

  my $oRunLane = npg_qc::api::run::lane->new({'util' => $oUtil,});

=head2 fields - returns an array of the fields in the run_tile table

  my $aFields = $oRunLane->fields();

=head2 get_tiles - arrayref containing run::lane::tile objects in tile order

  my $aRunLaneTiles = $oRunLane->get_tiles();

=head2 get_lane - accessor to obtain the lane number

  my $lane_number = $oRunLane->get_lane();

=head2 id_run - accessor to obtain the id_run (note this is the real id_run for the end)

  my $id_run = $oRunLaneTile->id_run();

=head2 get_read_end - returns 1 or 2 dependent whether this is the first or second end

=head2 create_tiles - run to set up run::lane::tile objects from an array of tiles for this run::lane

  $oRunLane->create_tiles($arrayref_tiles);

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Class::Std
Carp
npg_qc::api::run
Readonly

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
