#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-11-07
# Last Modified: $Date: 2008-11-12 12:31:52 +0000 (Wed, 12 Nov 2008) $
# Id:            $Id: lane.pm 3423 2008-11-12 12:31:52Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/lib/npg_qc/api/summary/lane.pm $
#
package npg_qc::api::summary::lane;
use strict;
use warnings;
use Class::Std;
use base qw(npg_qc::api::summary);

our $VERSION = do { my ($r) = q$LastChangedRevision: 3423 $ =~ /(\d+)/mx; $r; };

{
  ## no critic
  my %id_run_of                 :ATTR( :get<id_run>,                :set<id_run>                );
  my %lane_of                   :ATTR( :get<lane>,                  :set<lane>                  );
  my %yield_of                  :ATTR( :get<yield>,                 :set<yield>                 );
  my %total_raw_reads_of        :ATTR( :get<total_raw_reads>,       :set<total_raw_reads>       );
  my %total_pf_reads_of         :ATTR( :get<total_pf_reads>,        :set<total_pf_reads>        );
  my %clusters_raw_of           :ATTR( :get<clusters_raw>,          :set<clusters_raw>          );
  my %clusters_raw_sd_of        :ATTR( :get<clusters_raw_sd>,       :set<clusters_raw_sd>       );
  my %clusters_pf_of            :ATTR( :get<clusters_pf>,           :set<clusters_pf>           );
  my %clusters_pf_sd_of         :ATTR( :get<clusters_pf_sd>,        :set<clusters_pf_sd>        );
  my %first_cycle_int_of        :ATTR( :get<first_cycle_int>,       :set<first_cycle_int>       );
  my %first_cycle_int_sd_of     :ATTR( :get<first_cycle_int_sd>,    :set<first_cycle_int_sd>    );
  my %perc_int_20_cycles_of     :ATTR( :get<perc_int_20_cycles>,    :set<perc_int_20_cycles>    );
  my %perc_int_20_cycles_sd_of  :ATTR( :get<perc_int_20_cycles_sd>, :set<perc_int_20_cycles_sd> );
  my %perc_pf_clusters_of       :ATTR( :get<perc_pf_clusters>,      :set<perc_pf_clusters>      );
  my %perc_pf_clusters_sd_of    :ATTR( :get<perc_pf_clusters_sd>,   :set<perc_pf_clusters_sd>   );
  my %perc_pf_align_of          :ATTR( :get<perc_pf_align>,         :set<perc_pf_align>         );
  my %perc_pf_align_sd_of       :ATTR( :get<perc_pf_align_sd>,      :set<perc_pf_align_sd>      );
  my %align_score_of            :ATTR( :get<align_score>,           :set<align_score>           );
  my %align_score_sd_of         :ATTR( :get<align_score_sd>,        :set<align_score_sd>        );
  my %perc_error_rate_of        :ATTR( :get<perc_error_rate>,       :set<perc_error_rate>       );
  my %perc_error_rate_sd_of     :ATTR( :get<perc_error_rate_sd>,    :set<perc_error_rate_sd>    );
  ## use critic

  sub fields {
    return qw{
      id_run
      lane
      yield
      total_raw_reads
      total_pf_reads
      clusters_raw
      clusters_raw_sd
      clusters_pf
      clusters_pf_sd
      first_cycle_int
      first_cycle_int_sd
      perc_int_20_cycles
      perc_int_20_cycles_sd
      perc_pf_clusters
      perc_pf_clusters_sd
      perc_pf_align
      perc_pf_align_sd
      align_score
      align_score_sd
      perc_error_rate
      perc_error_rate_sd
    };
  }
}
1;
__END__

=head1 NAME

npg_qc::api::summary::lane

=head1 VERSION

$LastChangedRevision: 3423 $

=head1 SYNOPSIS

  my $oLaneSummary = npg_qc::api::summary::lane->new({'util' => $oUtil,});

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 fields - returns an array of the fields that populated for a Chip Summary

=head2 Accessors for obtaining the information within this object

=head2 get_id_run
=head2 get_lane
=head2 get_yield
=head2 get_total_raw_reads
=head2 get_total_pf_reads
=head2 get_clusters_raw
=head2 get_clusters_raw_sd
=head2 get_clusters_pf
=head2 get_clusters_pf_sd
=head2 get_first_cycle_int
=head2 get_first_cycle_int_sd
=head2 get_perc_int_20_cycles
=head2 get_perc_int_20_cycles_sd
=head2 get_perc_pf_clusters
=head2 get_perc_pf_clusters_sd
=head2 get_perc_pf_align
=head2 get_perc_pf_align_sd
=head2 get_align_score
=head2 get_align_score_sd
=head2 get_perc_error_rate
=head2 get_perc_error_rate_sd

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
Class::Std
npg_qc::api::summary

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut