#########
# Author:        ajb
# Maintainer:    $Author: ajb $
# Created:       2008-06-10
# Last Modified: $Date: 2008-07-31 14:57:23 +0100 (Thu, 31 Jul 2008) $
# Id:            $Id: tile_all.pm 2578 2008-07-31 13:57:23Z ajb $
# Source:        $Source: /repos/cvs/webcore/SHARED_docs/cgi-bin/docrep,v $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/lib/npg_qc/model/tile_all.pm $
#

package npg_qc::model::tile_all;
use strict;
use warnings;
use base qw(npg_qc::model);
use Readonly;

our $VERSION = do { my ($r) = q$LastChangedRevision: 2578 $ =~ /(\d+)/mx; $r; };

Readonly our $PERCENTAGE => 100;

__PACKAGE__->mk_accessors(__PACKAGE__->fields());

sub fields {
  return qw(
            id_tile_all
            id_run_tile
            all_a
            all_c
            all_g
            all_t
            call_a
            call_c
            call_g
            call_t
            num_a
            num_c
            num_g
            num_t
            num_x
            cycle
          );
}

sub run_tile {
  my $self  = shift;
  my $pkg   = 'npg_qc::model::run_tile';
  return $pkg->new({
		    'util' => $self->util(),
		    'id_run_tile' => $self->id_run_tile(),
		   });
}

sub calculate_signal_means {
  my ($self, $id_run) = @_;

  my $query = qq{SELECT ta.cycle, rt.id_run, rt.position, ta.cycle,
                        TRUNCATE(AVG(ta.all_a),2) AS all_a,
                        TRUNCATE(AVG(ta.all_c),2) AS all_c,
                        TRUNCATE(AVG(ta.all_g),2) AS all_g,
                        TRUNCATE(AVG(ta.all_t),2) AS all_t, 
                        TRUNCATE(AVG(ta.call_a),2) AS call_a,
                        TRUNCATE(AVG(ta.call_c),2) AS call_c,
                        TRUNCATE(AVG(ta.call_g),2) AS call_g,
                        TRUNCATE(AVG(ta.call_t),2) AS call_t, 
                        TRUNCATE((SUM(ta.num_a)*$PERCENTAGE/(SUM(ta.num_a)+SUM(ta.num_c)+SUM(ta.num_g)+SUM(ta.num_t))),2) AS base_a,
                        TRUNCATE((SUM(ta.num_c)*$PERCENTAGE/(SUM(ta.num_a)+SUM(ta.num_c)+SUM(ta.num_g)+SUM(ta.num_t))),2) AS base_c,
                        TRUNCATE((SUM(ta.num_g)*$PERCENTAGE/(SUM(ta.num_a)+SUM(ta.num_c)+SUM(ta.num_g)+SUM(ta.num_t))),2) AS base_g,
                        TRUNCATE((SUM(ta.num_t)*$PERCENTAGE/(SUM(ta.num_a)+SUM(ta.num_c)+SUM(ta.num_g)+SUM(ta.num_t))),2) AS base_t
                 FROM   run_tile rt,
                        tile_all ta
                 WHERE  rt.id_run = ?
                 AND    ta.id_run_tile = rt.id_run_tile
                 GROUP BY position, cycle};

  my $dbh = $self->util->dbh();
  my $sth = $dbh->prepare($query);
  $sth->execute($id_run);

  my $return_array;
  while (my $href = $sth->fetchrow_hashref()) {
    push @{$return_array}, $href;
  }

  return $return_array;
}

1;
__END__
=head1 NAME

npg_qc::model::tile_all

=head1 VERSION

$LastChangedRevision: 2578 $

=head1 SYNOPSIS

  my $oTileAll = npg_qc::model::tile_all->new({util => $util});

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 fields - return array of fields, first of which is the primary key

  my $aFields = $oTileAll->fields();

=head2 run_tile - returns run_tile object that this object belongs to

  my $oRunTile = $oTileAll->run_tile();

=head2 calculate_signal_means - returns the average signal means per cycle, given an id_run

  my $aCalculateSignalMeans = $oTileAll->calculate_signal_means($id_run);

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

strict
warnings
npg_qc::model

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Andy Brown, E<lt>ajb@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2008 GRL, by Andy Brown

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.

=cut
