#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2008-07-22
# Last Modified: $Date: 2008-12-01 11:14:26 +0000 (Mon, 01 Dec 2008) $
# Id:            $Id: 30-api-main_runlog.t 3636 2008-12-01 11:14:26Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/t/30-api-main_runlog.t $
#

use strict;
use warnings;
use Test::More tests => 14;
use English qw(-no_match_vars);
use IO::Scalar;
use lib ('t');
use SangerPaths qw(ensembl core badger trackingnpg tracking);
use t::api::util;
use npg::api::util;
use t::useragent;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3636 $ =~ /(\d+)/mx; $r; };

use_ok('npg_qc::api::main');

my $file_root            = q{t/data/staging/IL21/analysis/080530_IL21_0812/Data/C2-37_Firecrest1.9.2_03-06-2008_single/Bustard1.9.2_03-06-2008_single/GERALD_03-06-2008_single/};
my $config_url           = $file_root.q{config.txt};
my $tile_all             = $file_root.q{s_1_0001_all.txt};
my $tile_score           = $file_root.q{s_4_0061_score.txt};
my $tile_rescore         = $file_root.q{s_4_0061_rescore.txt};
my $run_signal_means     = $file_root.q{Signal_Means.txt};

{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/create_xml}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/create_xml.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict}    => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/9}  => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/9.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/12} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/12.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/17} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/17.xml},
    q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run_status_dict/18} => q{t/data/requests/cgi-bin/prodsoft/npg/npg/run_status_dict/18.xml},
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc//run_tile}      => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_tile_log.xml},
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc//run_tile/;list_movez_id_run_xml}      => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/movez_id_run.xml},
     q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/cache_query}   => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching.xml},
    
  };
  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});
  my $npg_api_util = npg::api::util->new({useragent => $ua});
  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/', npg_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg/npg/'});

  my $main_1 = npg_qc::api::main->new({util => $util});
  $main_1->set_runlog_file_root('t/data');
  $main_1->set_runlog_file_outgoing('staging/IL*/outgoing/*/RunLog_concat.xml');
  $main_1->set_runlog_file_analysis('staging/IL*/analysis/*/RunLog_concat.xml');
  
  #test for runs_done_runlog
  eval{ $main_1->runs_done_runlog(); };
  is($EVAL_ERROR, q{}, '$main->runs_done() did not croak');
  isa_ok($main_1->get_runs_done_runlog(), 'HASH', '$main->runs_done_runlog() stored');
  is(scalar keys%{$main_1->get_runs_done_runlog()}, 7, '$main->runs_done_runlog() has correct number of runs_done ids with row not null in stored hash');

  #test for retrieve_runlogfile_urls
  eval{$main_1->retrieve_runlogfile_urls();};
  is($EVAL_ERROR, q{}, '$main->retrieve_runlogfile_urls() did not croak');
  is(scalar @{$main_1->get_runlog_file_urls()}, 3, '$main->retrieve_runlogfile_urls() get the correct number of files'); 

  #test for retrieve_runlog_xml
  eval{$main_1->retrieve_runlog_xml();};
  is($EVAL_ERROR, q{}, '$main->retrieve_xml() did not croak');
  ok(scalar @{$main_1->get_runlog_output()}, '$main->retrieve_runlog_xml() is ok');
  ok($util->test_rendered($main_1->get_runlog_output()->[0], 't/data/rendered/move_z_1059.xml'), '$main->get_runlog_output()->[0] is correct');
 # ok($util->test_rendered($main_1->get_runlog_output()->[1], 't/data/rendered/move_z_859.xml'), '$main->get_runlog_output()->[1] is correct');

  #test for websend_runlog_xml
  eval{$main_1->websend_runlog_xml();};
  is($EVAL_ERROR, q{}, '$main->websend_runlog_xml() did not croak');
  is(scalar@{$main_1->get_response_objects()}, 3, '3 successful response objects for sending runlog xml');
  my $processed = $main_1->process_response_objects();
  isa_ok($processed, 'HASH', '$main->process_response_objects()');
  is($processed->{score}, 3, 'processed 3 response objects');

  #test for process_all_runlog
  my $main_2 = npg_qc::api::main->new({util => $util});
  $main_2->set_runlog_file_root('t/data');
  $main_2->set_runlog_file_outgoing('staging/IL*/outgoing/*/RunLog_concat.xml');
  $main_2->set_runlog_file_analysis('staging/IL*/analysis/*/RunLog_concat.xml');
  eval{
    $main_2->process_all_runlog();
  };
  is($EVAL_ERROR, q{}, '$main->process_all_runlog() did not croak');
 
  #is(scalar@{$main_2->get_response_objects()}, 2, '2 successful response objects for sending runlog xml');
}

1;
