#!/Users/ajb/dev/bin/perl -w
#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-06-12
# Last Modified: $Date: 2008-12-12 12:07:02 +0000 (Fri, 12 Dec 2008) $
# Id:            $Id: 30-api-process_bustard_output.t 3770 2008-12-12 12:07:02Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/t/30-api-process_bustard_output.t $
#

use strict;
use warnings;
use Test::More tests => 70;
use English qw(-no_match_vars);
use IO::Scalar;
use lib qw{t};
use SangerPaths qw(core ensembl);
use t::api::util;
use Perl6::Slurp;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3770 $ =~ /(\d+)/mx; $r; };

use_ok('npg_qc::api::process::bustard_output');
my $util = t::api::util->new({dbh => 'dummy', fixtures => 0, useragent => 0, parser => 0, base_uri => 0});

my $file_root            = q{t/data/staging/IL21/analysis/080530_IL21_0812/Data/C2-37_Firecrest1.9.2_03-06-2008_single/Bustard1.9.2_03-06-2008_single/GERALD_03-06-2008_single/};
my $config_url           = $file_root.q{config.txt};
my $config_content       = $util->rendered($config_url);
my $tile_all             = $file_root.q{s_1_0001_all.txt};
my $tile_all_content     = $util->rendered($tile_all);
my $tile_score           = $file_root.q{s_4_0061_score.txt};
my $tile_score_content   = $util->rendered($tile_score);
my $tile_rescore         = $file_root.q{s_4_0061_rescore.txt};
my $tile_rescore_content = $util->rendered($tile_rescore);
my $run_signal_means     = $file_root.q{Signal_Means.txt};
my $signal_means_content = $util->rendered($run_signal_means);
my $s1_signal_means      = $file_root.q{s_1_Signal_Means.txt};
my $summary              = q{t/data/staging/IL21/outgoing/080530_IL21_0812/Latest_Summary/Summary.htm};

{
  my $bustard_output = npg_qc::api::process::bustard_output->new({util => $util});
  isa_ok($bustard_output, 'npg_qc::api::process::bustard_output', '$bustard_output');
  is($bustard_output->file($config_url), $config_content, 'fetched the contents of the file');
  is($bustard_output->file(), $config_content, 'had cached contents of the file');
  $bustard_output->set_id_run(undef);
  is($bustard_output->id_run($config_url), 812, 'fetched the id_run');
  is($bustard_output->id_run(), 812, 'had cached the id_run');
  is($bustard_output->tile($tile_score), 61, 'fetched the tile');
  is($bustard_output->tile(), 61, 'had cached the tile');
  $bustard_output->set_position(undef);
  is($bustard_output->position($tile_all), 1, 'fetched the position');
  is($bustard_output->position(), 1, 'had cached the position');
  $bustard_output->set_rescore(undef);
  is($bustard_output->rescore($tile_rescore), 1, 'fetched rescore = 1');
  is($bustard_output->rescore(), 1, 'had cached rescore = 1');
  $bustard_output->set_rescore(undef);
  is($bustard_output->rescore($tile_score), 0, 'fetched rescore = 0');
  is($bustard_output->rescore(), 0, 'had cached rescore = 0');
}
{
  my $bustard_output = npg_qc::api::process::bustard_output->new({util => $util});
  my $config_test_file = slurp 't/data/rendered/bustard_config.xml';
  $config_test_file =~ s/\n\z//xm;
  is($bustard_output->config_xml($config_url), $config_test_file, '$bustard_output->config_xml($config_url) is correct');
  is($bustard_output->convert_to_xml($config_url), $config_test_file, '$bustard_output->convert_to_xml($config_url) is correct');
}
{
  my $bustard_output = npg_qc::api::process::bustard_output->new({util => $util});
  ok($util->test_rendered($bustard_output->tile_all_xml($tile_all), 't/data/rendered/tile_all.xml'), '$bustard_output->tile_all_xml($tile_all) is correct');
#test removed as no longer processing these files
 # ok($util->test_rendered($bustard_output->convert_to_xml($tile_all), 't/data/rendered/tile_all.xml'), '$bustard_output->convert_to_xml($tile_all) is correct');
}
{
  my $bustard_output = npg_qc::api::process::bustard_output->new({util => $util});
  ok($util->test_rendered($bustard_output->signal_means_xml($run_signal_means), 't/data/rendered/signal_means.xml'), '$bustard_output->signal_means_xml($run_signal_means) is correct');
  ok($util->test_rendered($bustard_output->convert_to_xml($run_signal_means), 't/data/rendered/signal_means.xml'), '$bustard_output->convert_to_xml($run_signal_means) is correct');
}
{
  my $bustard_output = npg_qc::api::process::bustard_output->new({util => $util});
  $bustard_output->set_rescore(0);
  ok($util->test_rendered($bustard_output->tile_score($tile_score_content), 't/data/rendered/tile_score.xml'), '$bustard_output->tile_score($tile_score_content) is correct');
  $bustard_output->set_rescore(1);
  ok($util->test_rendered($bustard_output->tile_score($tile_rescore_content), 't/data/rendered/tile_score_rescore.xml'), '$bustard_output->tile_score($tile_rescore_content) is correct');
}
{
  my $bustard_output = npg_qc::api::process::bustard_output->new({util => $util});
  my @lines = split "\n", $tile_score_content;
  my ($cumulative_errors_by_cycle, $log_likelihood, $information_content_by_cycle, $error_rate_relative_reference_cycle_nucleotide);
  foreach my $line (@lines) {
    my $processed = $line =~ /\A\!\d+/xms ? push @{$cumulative_errors_by_cycle}, $line
                  : $line =~ /\A\>\d+/xms ? push @{$log_likelihood}, $line
                  : $line =~ /\A\~\d+/xms ? push @{$information_content_by_cycle}, $line
                  : $line =~ /\A\@\d+/xms ? push @{$error_rate_relative_reference_cycle_nucleotide}, $line
                  :                         q{}
                  ;
  }
  $bustard_output->set_rescore(0);
  ok($util->test_rendered($bustard_output->cumulative_errors_by_cycle($cumulative_errors_by_cycle), 't/data/rendered/cumulative_errors_by_cycle.xml'), '$bustard_output->cumulative_errors_by_cycle($cumulative_errors_by_cycle) is correct');
  ok($util->test_rendered($bustard_output->log_likelihood($log_likelihood), 't/data/rendered/log_likelihood.xml'), '$bustard_output->log_likelihood($log_likelihood) is correct');
  ok($util->test_rendered($bustard_output->information_content_by_cycle($information_content_by_cycle), 't/data/rendered/information_content_by_cycle.xml'), '$bustard_output->information_content_by_cycle($information_content_by_cycle) is correct');
  ok($util->test_rendered($bustard_output->error_rate_relative_reference_cycle_nucleotide($error_rate_relative_reference_cycle_nucleotide), 't/data/rendered/error_rate_relative_reference_cycle_nucleotide.xml'), '$bustard_output->error_rate_relative_reference_cycle_nucleotide($error_rate_relative_reference_cycle_nucleotide) is correct');
  ok($util->test_rendered($bustard_output->errors_by_cycle($tile_score_content), 't/data/rendered/errors_by_cycle.xml'), '$bustard_output->errors_by_cycle($tile_score_content) is correct');
  ok($util->test_rendered($bustard_output->error_rate_relative_to_reference_base_inc_blanks($tile_score_content), 't/data/rendered/error_rate_relative_to_reference_base_inc_blanks.xml'), '$bustard_output->error_rate_relative_to_reference_base_inc_blanks($tile_score_content) is correct');
  ok($util->test_rendered($bustard_output->error_rate_relative_to_reference_base_no_blanks($tile_score_content), 't/data/rendered/error_rate_relative_to_reference_base_no_blanks.xml'), '$bustard_output->error_rate_relative_to_reference_base_no_blanks($tile_score_content) is correct');
  ok($util->test_rendered($bustard_output->error_rate_relative_sequence_base($tile_score_content), 't/data/rendered/error_rate_relative_sequence_base.xml'), '$bustard_output->error_rate_relative_sequence_base($tile_score_content) is correct');
  ok($util->test_rendered($bustard_output->errors_by_nucleotide($tile_score_content), 't/data/rendered/errors_by_nucleotide.xml'), '$bustard_output->errors_by_nucleotide($tile_score_content) is correct');
  ok($util->test_rendered($bustard_output->errors_by_cycle_and_nucleotide($tile_score_content), 't/data/rendered/errors_by_cycle_and_nucleotide.xml'), '$bustard_output->errors_by_cycle_and_nucleotide($tile_score_content) is correct');
  ok($util->test_rendered($bustard_output->most_common_word($tile_score_content), 't/data/rendered/most_common_word.xml'), '$bustard_output->most_common_word($tile_score_content) is correct');
  ok($util->test_rendered($bustard_output->most_common_blank_pattern($tile_score_content), 't/data/rendered/most_common_blank_pattern.xml'), '$bustard_output->most_common_blank_pattern($tile_score_content) is correct');
  ok($util->test_rendered($bustard_output->tile_score_xml($tile_rescore), 't/data/rendered/tile_score_xml.xml'), '$bustard_output->tile_score_xml($tile_rescore) is correct');
  ok($util->test_rendered($bustard_output->convert_to_xml($tile_rescore), 't/data/rendered/tile_score_xml.xml'), '$bustard_output->convert_to_xml($tile_rescore) is correct');
}
{
  my $bustard_output = npg_qc::api::process::bustard_output->new({util => $util});
  my $r = $bustard_output->convert_to_xml($s1_signal_means);
  is($r, undef, 'nothing returned, as not correct SignalMeans file');
}
{
  my $bustard_output = npg_qc::api::process::bustard_output->new({util => $util});
  my $xml;
  eval{ $xml = $bustard_output->convert_to_xml($summary); };
  is($EVAL_ERROR, q{}, 'no croak on $bustard_output->convert_to_xml($summary)');
  my $parser = $util->parser();
  my $parsed_xml = $parser->parse_string($xml);
  is($parsed_xml->getElementsByTagName('dataset'), 'summary_data', q{$parsed_xml->getElementsByTagName('dataset') is correct});
  my @chip_summary = $parsed_xml->getElementsByTagName('chip_summary');
  my @rows_chip_summary = $chip_summary[0]->getElementsByTagName('row');
  is($rows_chip_summary[0]->getAttribute('clusters_pf'), 109999208, 'clusters_pf is correct');
  is($rows_chip_summary[0]->getAttribute('yield_kb'), 3959971, 'yield_kb is correct');
  my @lanes = $parsed_xml->getElementsByTagName('lane');
  my @rows_for_lane_3_read_2 = $lanes[5]->getElementsByTagName('row');
  is($rows_for_lane_3_read_2[19]->getAttribute('end'), 2, 'end (read) is correct');
  is($rows_for_lane_3_read_2[19]->getAttribute('lane'), 3, 'lane is correct');
  is($rows_for_lane_3_read_2[19]->getAttribute('tile'), '0020', 'tile is correct');
  is($rows_for_lane_3_read_2[19]->getAttribute('av_perc_intensity_after_20_cycles_pf'), '74.30', 'av_perc_intensity_after_20_cycles_pf is correct');
  ok($util->test_rendered($xml, 't/data/rendered/summary.xml'), '$bustard_output->convert_to_xml($summary) is correct');
}
#__END__
$file_root	      = q{t/data/staging/IL27/analysis/080620_IL27_0926/Data/C1-74_Firecrest1.9.5_28-06-2008_auto/Bustard1.9.5_28-06-2008_auto/GERALD1.9.5_28-06-2008_auto/};
$tile_score	      = $file_root.q{s_4_0235_score.txt};
$tile_score_content   = $util->rendered($tile_score);
$tile_rescore	      = $file_root.q{s_4_0235_rescore.txt};
$tile_rescore_content = $util->rendered($tile_rescore);
$run_signal_means     = $file_root.q{Signal_Means.txt};
$signal_means_content = $util->rendered($run_signal_means);
{
  my $bustard_output = npg_qc::api::process::bustard_output->new({util => $util});
  my $xml;
  eval{ $xml = $bustard_output->convert_to_xml($tile_score); };
  ok($util->test_rendered($xml, 't/data/rendered/paired_tile_score.xml'), '$bustard_output->convert_to_xml($tile_score_paired) is correct');
  my $parser     = $util->parser();
  my $parsed_xml = $parser->parse_string($xml);
  is($parsed_xml->getElementsByTagName('dataset'), 'tile_score', q{$parsed_xml->getElementsByTagName('dataset') is correct});
  is(scalar@{$parsed_xml->getElementsByTagName('run_tile')}, 2, q{Array at $parsed_xml->getElementsByTagName('run_tile') has correct number of elements});
  my $count = 1;
  foreach my $read (@{$parsed_xml->getElementsByTagName('run_tile')}) {
    if ($count == 1) {
      is($read->getElementsByTagName('tile_score')->shift()->getAttribute('base_count'), 1053094, "base_count read$count is correct");
      my $temp = $read->getElementsByTagName('cumulative_errors_by_cycle')->shift()->getElementsByTagName('row');
      is($temp->[5]->getAttribute('three'), 28460, 'cumulative_errors_by_cycle, cycle 6, attribute 3');
      $temp = $read->getElementsByTagName('log_likelihood')->shift()->getElementsByTagName('row');
      is($temp->[26]->getAttribute('really_t'), '-264', 'log_likelihood, cycle 6, read_as A, really_t');
      $temp = $read->getElementsByTagName('errors_by_nucleotide')->shift()->getElementsByTagName('row');
      is($temp->[0]->getAttribute('read_as'), 'N', 'errors_by_nucleotide, first row, read_as');
    } else {
      is($read->getElementsByTagName('tile_score')->shift()->getAttribute('blank_count'), 131, "blank_count read$count is correct");
      is($read->getElementsByTagName('tile_score')->shift()->getAttribute('score_date_run'), '2008-06-28 20:52:21', "score_date_run read$count is correct");
      my $temp = $read->getElementsByTagName('information_content_by_cycle')->shift()->getElementsByTagName('row');
      is($temp->[33]->getAttribute('equiv_info'), '21412.41', 'information_content_by_cycle, cycle 34, equiv_info');
      $temp = $read->getElementsByTagName('error_rate_relative_reference_cycle_nucleotide')->shift()->getElementsByTagName('row');
      is($temp->[179]->getAttribute('really_a'), '0.014', 'error_rate_relative_reference_cycle_nucleotide, row 180, really_a');
      $temp = $read->getElementsByTagName('errors_by_cycle_and_nucleotide')->shift()->getElementsByTagName('row');
      is($temp->[12]->getAttribute('really_c'), 5900, 'errors_by_cycle_and_nucleotide, row 13, really_c');
    }
    $count++;
  }
}
my $qcal_end_1 = $file_root.q{s_8_1_qcalreport.txt};
my $qcal_end_2 = $file_root.q{s_8_2_qcalreport.txt};
{
  my $bustard_output = npg_qc::api::process::bustard_output->new({util => $util});
  my $xml;
  eval{ $xml = $bustard_output->convert_to_xml($qcal_end_1); };
  is($EVAL_ERROR, q{}, q{didn't croak on converting s_8_1_qcalreport.txt to xml});
  ok($util->test_rendered($xml, q{t/data/rendered/qcal_end_1.xml}), 'xml rendered ok');
  $bustard_output = npg_qc::api::process::bustard_output->new({util => $util});
  eval{ $xml = $bustard_output->convert_to_xml($qcal_end_2); };
  is($EVAL_ERROR, q{}, q{didn't croak on converting s_8_2_qcalreport.txt to xml});
  ok($util->test_rendered($xml, q{t/data/rendered/qcal_end_2.xml}), 'xml rendered ok');
}
$file_root	         = q{t/data/staging/IL27/analysis/080620_IL27_0926/Data/C1-74_Firecrest1.9.5_28-06-2008_auto/archive/};
my $qcal_non_control = $file_root.q{926_s_6.fastqcheck};
{
  my $bustard_output = npg_qc::api::process::bustard_output->new({util => $util});
  my $xml;
  eval{ $xml = $bustard_output->convert_to_xml($qcal_non_control); };
  is($EVAL_ERROR, q{}, q{didn't croak on converting 926_s_6.fastqcheck to xml'});
  ok($util->test_rendered($xml, q{t/data/rendered/qcal_from_fastqcheck.xml}), q{xml rendered ok});
}
$qcal_non_control = $file_root.q{926_s_6_2.fastqcheck};
{
  my $bustard_output = npg_qc::api::process::bustard_output->new({util => $util});
  my $xml;
  eval{ $xml = $bustard_output->convert_to_xml($qcal_non_control); };
  is($EVAL_ERROR, q{}, q{didn't croak on converting 926_s_6_2.fastqcheck to xml'});
  ok($util->test_rendered($xml, q{t/data/rendered/qcal_from_fastqcheck_2.xml}), q{xml rendered ok});
}

# release candidate 1.9.6 which was not ever expected to be used unless by specific request

$file_root	      = q{t/data/staging/IL1/outgoing/081203_IL1_1670/Data/C1-74_firecrest1.9.6_19-11-2008_auto/Bustard1.9.6_19-11-2008_auto/GERALD_19-11-2008_auto/};
$tile_score	      = $file_root.q{s_4_0100_score.txt};
$tile_score_content   = $util->rendered($tile_score);
$tile_rescore	      = $file_root.q{s_4_0100_rescore.txt};
$tile_rescore_content = $util->rendered($tile_rescore);
{
  my $bustard_output = npg_qc::api::process::bustard_output->new({util => $util});
  my $xml;
  eval{ $xml = $bustard_output->convert_to_xml($tile_score); };
  ok($util->test_rendered($xml, 't/data/rendered/tile_score_1_9_6.xml'), '$bustard_output->convert_to_xml(tile_score_1_9_6) is correct');
  $bustard_output = npg_qc::api::process::bustard_output->new({util => $util});
  eval{ $xml = $bustard_output->convert_to_xml($tile_rescore); };
  ok($util->test_rendered($xml, 't/data/rendered/tile_rescore_1_9_6.xml'), '$bustard_output->convert_to_xml(tile_rescore_1_9_6) is correct');
}

# release candidate 1.3 expected to enter production sometime in the future

$file_root	      = q{t/data/staging/IL1/outgoing/081203_IL1_1829/Data/C1-74_Firecrest1.3rc1_28-11-2008_prelim/Bustard1.3rc1_28-11-2008_prelim/GERALD_28-11-2008_prelim/};
$tile_score	      = $file_root.q{s_4_0001_score.txt};
$tile_score_content   = $util->rendered($tile_score);
$tile_rescore	      = $file_root.q{s_4_0001_rescore.txt};
$tile_rescore_content = $util->rendered($tile_rescore);
$summary              = $file_root.q{Summary.htm};

{
  my $bustard_output = npg_qc::api::process::bustard_output->new({util => $util});
  my $xml;
  eval{ $xml = $bustard_output->convert_to_xml($tile_score); };
  ok($util->test_rendered($xml, 't/data/rendered/tile_score_rc1_3.xml'), '$bustard_output->convert_to_xml(tile_score_rc1_3) is correct');
}
{
  my $bustard_output = npg_qc::api::process::bustard_output->new({util => $util});
  my $xml;
  eval{ $xml = $bustard_output->convert_to_xml($tile_rescore); };
  ok($util->test_rendered($xml, 't/data/rendered/tile_rescore_rc1_3.xml'), '$bustard_output->convert_to_xml(tile_rescore_rc1_3) is correct');
}
{
  my $bustard_output = npg_qc::api::process::bustard_output->new({util => $util});
  my $xml;
  eval{ $xml = $bustard_output->convert_to_xml($summary); };
#  warn "\n\n\n\n\n\n\n\n\n$xml\n\n\n\n\n\n\n\n\n";
  ok($util->test_rendered($xml, 't/data/rendered/summary_rc1_3.xml'), '$bustard_output->convert_to_xml(summary_rc1_3) is correct');
}

1;
