#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2008-09-30
# Last Modified: $Date: 2008-12-01 11:14:26 +0000 (Mon, 01 Dec 2008) $
# Id:            $Id: 30-api-run_caching.t 3636 2008-12-01 11:14:26Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/t/30-api-run_caching.t $
#

use strict;
use warnings;
use Test::More  tests => 19;
use English qw(-no_match_vars);
use lib ('t');
use SangerPaths qw(ensembl core badger trackingnpg tracking);
use t::api::util;
use npg::api::util;
use t::useragent;

our $VERSION = do { my ($r) = q$LastChangedRevision: 3636 $ =~ /(\d+)/mx; $r; };
###
use_ok('npg_qc::api::run_caching');

###
{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/run_graph}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching.xml},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});

  my $npg_api_util = npg::api::util->new({useragent => $ua});

  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/'});

  my $api_run_caching = npg_qc::api::run_caching->new({util => $util});

  isa_ok($api_run_caching, 'npg_qc::api::run_caching', '$api_run_caching model');

  eval{
    my $xml = q{<?xml version="1.0" encoding="utf-8"?><root><method>run_graph</method></root>};
    my $web_target = q{run_graph};
    $api_run_caching->web_request_by_xml($xml, $web_target);
    1;
  };
  is($EVAL_ERROR, q{}, 'web_request_by_xml did not croak');
}

###
{
  my $ua_mock = {};
  my $ua   = t::useragent->new({is_success => 0, mock => $ua_mock});
  my $util = t::api::util->new({dbh => 'dummy', fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/'});

  my $api_run_caching = npg_qc::api::run_caching->new({util => $util});

  eval{
    my $xml = q{<?xml version="1.0" encoding="utf-8"?><root><method>run_graph</method></root>};
    my $web_target = q{run_graph};
    $api_run_caching->web_request_by_xml($xml, $web_target);
    1;
  };
  like($EVAL_ERROR, qr{failed\ to\ successfully\ post}, 'web_request_by_xml time out');
}

###
{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/run_graph}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching_fail.xml},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});

  my $npg_api_util = npg::api::util->new({useragent => $ua});

  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/'});

  my $api_run_caching = npg_qc::api::run_caching->new({util => $util});

  eval{
    my $xml = q{<?xml version="1.0" encoding="utf-8"?><root><method>run_graph</method></root>};
    my $web_target = q{run_graph};
    $api_run_caching->web_request_by_xml($xml, $web_target);
    1;
  };
  like($EVAL_ERROR, qr{request\ not\ successfully}, 'web_request_by_xml not successfully');
}


###
{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/run_graph}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching.xml},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});

  my $npg_api_util = npg::api::util->new({useragent => $ua});

  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/'});

  my $api_run_caching = npg_qc::api::run_caching->new({util => $util});

  eval{

    $api_run_caching->run_graph();
    1;
  };
  is($EVAL_ERROR, q{}, 'run_graph did not croak');
}
###
{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/run_graph}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching.xml},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});

  my $npg_api_util = npg::api::util->new({useragent => $ua});

  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/'});

  my $api_run_caching = npg_qc::api::run_caching->new({util => $util});

  eval{

    $api_run_caching->run_graph(10, 2);
    1;
  };
  is($EVAL_ERROR, q{}, 'run_graph with args did not croak');
}
###
{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/run_graph}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching_fail.xml},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});

  my $npg_api_util = npg::api::util->new({useragent => $ua});

  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/'});

  my $api_run_caching = npg_qc::api::run_caching->new({util => $util});

  eval{
    $api_run_caching->run_graph();
    1;
  };
  like($EVAL_ERROR, qr{request\ not\ successfully}, 'run_graph not successfully');
}

###
{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/cache_query}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching.xml},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});

  my $npg_api_util = npg::api::util->new({useragent => $ua});

  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/'});

  my $api_run_caching = npg_qc::api::run_caching->new({util => $util});

  eval{

    $api_run_caching->cache_query_movez_tiles();
    1;
  };
  is($EVAL_ERROR, q{}, 'cache_query_movez_tiles did not croak');
}

###
{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/cache_query}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching_fail.xml},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});

  my $npg_api_util = npg::api::util->new({useragent => $ua});

  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/'});

  my $api_run_caching = npg_qc::api::run_caching->new({util => $util});

  eval{
    $api_run_caching->cache_query_movez_tiles();
    1;
  };
  like($EVAL_ERROR, qr{request\ not\ successfully}, 'cache_query_movez_tiles not successfully');
}

###
{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/cache_query}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching.xml},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});

  my $npg_api_util = npg::api::util->new({useragent => $ua});

  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/'});

  my $api_run_caching = npg_qc::api::run_caching->new({util => $util});

  eval{

    $api_run_caching->cache_query_lane_summary();
    1;
  };
  is($EVAL_ERROR, q{}, 'cache_query_lane_summary did not croak');
}

###
{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/cache_query}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching_fail.xml},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});

  my $npg_api_util = npg::api::util->new({useragent => $ua});

  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/'});

  my $api_run_caching = npg_qc::api::run_caching->new({util => $util});

  eval{
    $api_run_caching->cache_query_lane_summary();
    1;
  };
  like($EVAL_ERROR, qr{request\ not\ successfully}, 'cache_query_lane_summary not successfully');
}

{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/cache_query}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching.xml},
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/run_graph}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching.xml},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});

  my $npg_api_util = npg::api::util->new({useragent => $ua});

  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/'});

  my $api_run_caching = npg_qc::api::run_caching->new({util => $util});

  eval{
    $api_run_caching->cache_all();
    1;
  };
  is($EVAL_ERROR, q{}, 'cache_all successfully');
}

###
{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/cache_query}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching.xml},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});

  my $npg_api_util = npg::api::util->new({useragent => $ua});

  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/'});

  my $api_run_caching = npg_qc::api::run_caching->new({util => $util});

  eval{

    $api_run_caching->cache_query_movez_tiles(10, 1);
    1;
  };
  is($EVAL_ERROR, q{}, 'cache_query_movez_tiles with args did not croak');
}

###
{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/cache_query}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching.xml},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});

  my $npg_api_util = npg::api::util->new({useragent => $ua});

  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/'});

  my $api_run_caching = npg_qc::api::run_caching->new({util => $util});

  eval{

    $api_run_caching->cache_query_lane_summary(10, 1, 1);
    1;
  };
  is($EVAL_ERROR, q{}, 'cache_query_lane_summary with args did not croak');
}

###
{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/cache_query}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching.xml},
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/run_graph}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching.xml},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});

  my $npg_api_util = npg::api::util->new({useragent => $ua});

  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/'});

  my $api_run_caching = npg_qc::api::run_caching->new({util => $util});

  eval{

    $api_run_caching->cache_all(10, 1, 0);
    1;
  };
  is($EVAL_ERROR, q{}, 'cache_call with args did not croak');
}


###
{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/cache_query}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching.xml},
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/run_graph}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching.xml},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});

  my $npg_api_util = npg::api::util->new({useragent => $ua});

  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/'});

  my $api_run_caching = npg_qc::api::run_caching->new({util => $util});

  eval{
    my $list_runs = [15, 16];
    $api_run_caching->cache_list_query_lane_summary($list_runs);
    1;
  };
  is($EVAL_ERROR, q{}, 'cache list query lane summary did not croak');
}

###
{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/cache_query}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching.xml},
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/run_graph}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching.xml},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});

  my $npg_api_util = npg::api::util->new({useragent => $ua});

  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/'});

  my $api_run_caching = npg_qc::api::run_caching->new({util => $util});

  eval{
    my $list_runs = [15, 16];
    $api_run_caching->cache_list_query_movez_tiles($list_runs);
    1;
  };
  is($EVAL_ERROR, q{}, 'cache list query movez_tiles did not croak');
}


###
{
  my $ua_mock = {
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/cache_query}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching.xml},
    q{http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/run_graph}     => q{t/data/requests/cgi-bin/prodsoft/npg_qc/npg_qc/run_caching.xml},
  };

  my $ua   = t::useragent->new({is_success => 1, mock => $ua_mock});

  my $npg_api_util = npg::api::util->new({useragent => $ua});

  my $util = t::api::util->new({dbh => 'dummy', npg_api_util => $npg_api_util, fixtures => 0, useragent => $ua, parser => 0, base_uri => 'http://localhost:8080/cgi-bin/prodsoft/npg_qc/npg_qc/'});

  my $api_run_caching = npg_qc::api::run_caching->new({util => $util});

  eval{
    my $list_runs = [15, 16];
    $api_run_caching->cache_list_query_run_graph($list_runs);
    1;
  };
  is($EVAL_ERROR, q{}, 'cache list query run graph did not croak');
}
1;
