#########
# Author:        ajb
# Maintainer:    $Author: gq1 $
# Created:       2008-12-08
# Last Modified: $Date: 2008-12-12 12:07:02 +0000 (Fri, 12 Dec 2008) $
# Id:            $Id: 40-api-util.t 3770 2008-12-12 12:07:02Z gq1 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-qc/trunk/t/40-api-util.t $
#

use strict;
use warnings;
use Test::More tests => 25;
use English qw(-no_match_vars);
use SangerPaths qw(core tracking badger ensembl);
use IO::Scalar;
use lib qw{t};
use Perl6::Slurp;
our $VERSION = do { my ($r) = q$LastChangedRevision: 3770 $ =~ /(\d+)/mx; $r; };

use_ok('npg_qc::api::util');

my $config = slurp('data/config.ini');

{
  my $api_util = npg_qc::api::util->new({
    base_uri     => undef,
    npg_uri      => undef,
    useragent    => undef,
    parser       => undef,
    npg_api_util => undef,
  });

  isa_ok($api_util, 'npg_qc::api::util', '$api_util');
  is($api_util->get_base_uri(), q{http://intweb.sanger.ac.uk/cgi-bin/prodsoft/npg_qc/npg_qc/}, '$api_util->get_base_uri() default ok');
  is($api_util->get_npg_uri(), q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/}, '$api_util->get_npg_uri() default ok');
  isa_ok($api_util->get_parser(), q{XML::LibXML}, '$api_util->get_parser() default');
  isa_ok($api_util->get_npg_api_util(), q{npg::api::util}, '$api_util->get_npg_api_util() default');
  is($api_util->get_useragent(), undef, 'no useragent set by default');

  is($api_util->dbsection(), 'live', 'live returned from dbsection');

  my ($live_block) = $config =~ /\[live\]\n(.*?)\n\n\[dev\]/xms;
  my @creds = split /\n/xms, $live_block;
  my %credentials;

  foreach my $c (@creds) {
    my @temp = split /=/xms, $c;
    $credentials{$temp[0]} = $temp[1];
  }

  is($api_util->dbname(), $credentials{dbname}, 'dbname retrieved from file ok');
  is($api_util->dbhost(), $credentials{dbhost}, 'dbhost retrieved from file ok');
  is($api_util->dbuser(), $credentials{dbuser}, 'dbuser retrieved from file ok');
  is($api_util->dbport(), $credentials{dbport}, 'dbport retrieved from file ok');
  is($api_util->dbpass(), $credentials{dbpass}, 'dbpass retrieved from file ok');

  my $dbh = $api_util->dbh();
  isa_ok($dbh, 'DBI::db', '$api_util->dbh() connected ok');
  is($api_util->dbh(), $dbh, 'dbh cached ok');
}

$ENV{dev} = 'test';

{
  my $api_util = npg_qc::api::util->new({
    base_uri     => undef,
    npg_uri      => undef,
    useragent    => undef,
    parser       => undef,
    npg_api_util => undef,
  });

  isa_ok($api_util, 'npg_qc::api::util', '$api_util');

  is($api_util->get_dbh(), undef, 'new util, new empty cache of dbh');
  is($api_util->dbsection(), 'test', 'test returned from dbsection');

  my ($test_block) = $config =~ /\[test\]\n(.*?)\z/xms;

  my @creds = split /\n/xms, $test_block;
  my %credentials;

  foreach my $c (@creds) {
    my @temp = split /=/xms, $c;
    $credentials{$temp[0]} = $temp[1];
  }

  is($api_util->dbname(), $credentials{dbname}, 'dbname retrieved from file ok');
  is($api_util->dbhost(), $credentials{dbhost}, 'dbhost retrieved from file ok');
  is($api_util->dbuser(), $credentials{dbuser}, 'dbuser retrieved from file ok');
  is($api_util->dbport(), $credentials{dbport}, 'dbport retrieved from file ok');
  is($api_util->dbpass(), $credentials{dbpass}, 'dbpass retrieved from file ok');

  my $dbh = $api_util->dbh();
  isa_ok($dbh, 'DBI::db', '$api_util->dbh() connected ok');
  is($api_util->dbh(), $dbh, 'dbh cached ok');
}
